/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2015 All rights reserved.                  */
/*  File Name    : SGTK.h                                                    */
/*  Abstract     : I/O toolkit header file                                   */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2015/03/20                                                    */
/*  Version    1.00                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#ifndef _SGTK_H_
#define _SGTK_H_

#include <stdint.h>
#include <string>
#include <map>
#include <hdf5.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned int uint;

/*****************************************************************************/
/*    Function prototypes                                                    */
/*****************************************************************************/
//hid_t SGTK_open_file(char *file_name, uint mode);
hid_t SGTK_open_file(char *file_name);
int SGTK_close_file(hid_t HDF_file_id);
int SGTK_get_dims(hid_t HDF_file_id, char *label, int **num);
int SGTK_get_dataset_int(hid_t HDF_file_id, char *access_label, int **get_value);
int SGTK_get_dataset_int8(hid_t HDF_file_id, char *label, int8_t **get_value);
int SGTK_get_dataset_int16(hid_t HDF_file_id, char *label, int16_t **get_value);
int SGTK_get_dataset_int32(hid_t HDF_file_id, char *label, int32_t **get_value);
int SGTK_get_dataset_uint(hid_t HDF_file_id, char *label, uint **get_value);
int SGTK_get_dataset_uint8(hid_t HDF_file_id, char *label, uint8_t **get_value);
int SGTK_get_dataset_uint16(hid_t HDF_file_id, char *label, uint16_t **get_value);
int SGTK_get_dataset_uint32(hid_t HDF_file_id, char *label, uint32_t **get_value);
int SGTK_get_dataset_float(hid_t HDF_file_id, char *label, int kind, float **get_value);
int SGTK_get_dataset_float32(hid_t HDF_file_id, char *label, int kind, float **get_value);
int SGTK_get_dataset_float64(hid_t HDF_file_id, char *label, int kind, double **get_value);
int SGTK_get_dataset_latlon(hid_t HDF_file_id, char *label_lat, char *label_lon, int kind, int start_no, int end_no, float **latlon);

int SGTK_set_dims(hid_t HDF_file_id, char *label, int *dims, int rank, int dat_type, char **dim_lbl);

int SGTK_set_dataset_int(hid_t HDF_file_id, char *label, int *dims, int rank, int *value);
int SGTK_set_dataset_int8(hid_t HDF_file_id, char *label, int *dims, int rank, int8_t *value);
int SGTK_set_dataset_int16(hid_t HDF_file_id, char *label, int *dims, int rank, int16_t *value);
int SGTK_set_dataset_int32(hid_t HDF_file_id, char *label, int *dims, int rank, int32_t *value);
int SGTK_set_dataset_uint(hid_t HDF_file_id, char *label, int *dims, int rank, uint *value);
int SGTK_set_dataset_uint8(hid_t HDF_file_id, char *label, int *dims, int rank, uint8_t *value);
int SGTK_set_dataset_uint16(hid_t HDF_file_id, char *label, int *dims, int rank, uint16_t *value);
int SGTK_set_dataset_uint32(hid_t HDF_file_id, char *label, int *dims, int rank, uint32_t *value);

int SGTK_set_dataset_float(hid_t HDF_file_id, char *label, int mode, int *dims, int rank, float *value);
int SGTK_set_dataset_float32(hid_t HDF_file_id, char *label, int mode, int *dims, int rank, float *value);
int SGTK_set_dataset_float64(hid_t HDF_file_id, char *label, int mode, int *dims, int rank, double *value);
int SGTK_set_dataset_latlon(hid_t HDF_file_id, char *label_lat, char *label_lon, int kind, int start_no, int end_no, int set_num, float *latlon);

int SGTK_get_meta_string(hid_t HDF_file_id, char *label, char *meta, char *value);
int SGTK_get_meta_int(hid_t HDF_file_id, char *label, char *meta, int *value);
int SGTK_get_meta_int8(hid_t HDF_file_id, char *label, char *meta, int8_t *value);
int SGTK_get_meta_int16(hid_t HDF_file_id, char *label, char *meta, int16_t *value);
int SGTK_get_meta_int32(hid_t HDF_file_id, char *label, char *meta, int32_t *value);
int SGTK_get_meta_uint(hid_t HDF_file_id, char *label, char *meta, uint *value);
int SGTK_get_meta_uint8(hid_t HDF_file_id, char *label, char *meta, uint8_t *value);
int SGTK_get_meta_uint16(hid_t HDF_file_id, char *label, char *meta, uint16_t *value);
int SGTK_get_meta_uint32(hid_t HDF_file_id, char *label, char *meta, uint32_t *value);
int SGTK_get_meta_float(hid_t HDF_file_id, char *label, char *meta, float *value);
int SGTK_get_meta_float32(hid_t HDF_file_id, char *label, char *meta, float *value);
int SGTK_get_meta_float64(hid_t HDF_file_id, char *label, char *meta, double *value);

int SGTK_set_meta_string(hid_t HDF_file_id, char *label, char *meta, char *value);
int SGTK_set_meta_int(hid_t HDF_file_id, char *label, char *meta, int value);
int SGTK_set_meta_int8(hid_t HDF_file_id, char *label, char *meta, int8_t value);
int SGTK_set_meta_int16(hid_t HDF_file_id, char *label, char *meta, int16_t value);
int SGTK_set_meta_int32(hid_t HDF_file_id, char *label, char *meta, int32_t value);
int SGTK_set_meta_uint(hid_t HDF_file_id, char *label, char *meta, uint value);
int SGTK_set_meta_uint8(hid_t HDF_file_id, char *label, char *meta, uint8_t value);
int SGTK_set_meta_uint16(hid_t HDF_file_id, char *label, char *meta, uint16_t value);
int SGTK_set_meta_uint32(hid_t HDF_file_id, char *label, char *meta, uint32_t value);
int SGTK_set_meta_float(hid_t HDF_file_id, char *label, char *meta, float value);
int SGTK_set_meta_float32(hid_t HDF_file_id, char *label, char *meta, float value);
int SGTK_set_meta_float64(hid_t HDF_file_id, char *label, char *meta, double value);

int SGTK_make_group(hid_t HDF_file_id, char *group);
int SGTK_utc_to_tai93(int year, int month, int day, int hour, int min, int sec, int msec, double *tai93time);
int SGTK_tai93_to_utc(double tai93time, int *year, int *month, int *day, int *hour, int *min, int *sec, int *msec);
void *SGTK_malloc(int *dims, int rank, int size);
int SGTK_free(void *buf);
int SGTK_get_global_latlon(hid_t HDF_file_id, float *latlon);
int SGTK_load_config(char *file_name);

int SGTK_make_parameter_table( const std::string parameter_file );
int SGKT_get_label_to_path(const std::string label, std::string& path);
	
// O[oϐ
extern std::map<std::string, std::string> SGLI_ACCESS_LABEL_TABLE;

// TEST
void SGTK_test(hid_t HDF_file_id);

/*****************************************************************************/
/* data type define                                                          */
/*****************************************************************************/
#define     SGTK_H5T_STD_I8_TYPE            (0)     /* char type            */
#define     SGTK_H5T_STD_U8_TYPE            (1)     /* unsigned char type   */
#define     SGTK_H5T_STD_I16_TYPE           (2)     /* short type           */
#define     SGTK_H5T_STD_U16_TYPE           (3)     /* unsgined short type  */
#define     SGTK_H5T_STD_I32_TYPE           (4)     /* int type             */
#define     SGTK_H5T_STD_U32_TYPE           (5)     /* unsgined int type    */
#define     SGTK_H5T_STD_I64_TYPE           (6)     /* long type            */
#define     SGTK_H5T_STD_U64_TYPE           (7)     /* unsigned long        */
#define     SGTK_H5T_IEEE_F32_TYPE          (8)     /* float type           */
#define     SGTK_H5T_IEEE_F64_TYPE          (9)     /* double type          */
#define     SGTK_H5T_C_S1_TYPE              (10)    /* string type          */

/*****************************************************************************/
/* kind of value (radiance/reflectance/other).                               */
/*****************************************************************************/
#define     SGTK_KIND_RADIANCE              (0)
#define     SGTK_KIND_REFLECTANCE           (1)
#define     SGTK_KIND_OTHER                 (3)

/*****************************************************************************/
/* file open mode.                                                           */
/*****************************************************************************/
#define     SGTK_HDF_ACC_RDONLY             (H5F_ACC_RDONLY)
#define     SGTK_HDF_ACC_RDWR               (H5F_ACC_RDWR)


#ifdef __cplusplus
}
#endif

#endif
