/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2015 All rights reserved.                  */
/*  File Name    : SGTK_cnf.h                                                */
/*  Abstract     : I/O toolkit header file                                   */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2015/03/20                                                    */
/*  Version    1.00                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#ifndef __SGTK_CONF__
#define __SGTK_CONF__

#include <vector>
#include <map>
#include <string>
#include "H5Cpp.h"
//#include "SGTK_hdf.h"

#ifdef _LINUX_
// Linux path
#define		SGTK_LEAP_SEC_FILE_PATH		"../../params/leapsec.dat"
#define		SGTK_EQR_LATLON_FILE_PATH	"../../params/eqr_latlon.bin"
#define		SGTK_EQA_LATLON_FILE_PATH	"../../params/eqa_latlon.bin"
#define		SGTK_PS_LATLON_FILE_PATH	"../../params/ps_latlon.bin"
#else
// Windows path
#define		SGTK_LEAP_SEC_FILE_PATH		"..\\..\\params\\leapsec.dat"
#define		SGTK_EQR_LATLON_FILE_PATH	"..\\..\\params\\eqr_latlon.bin"
#define		SGTK_EQA_LATLON_FILE_PATH	"..\\..\\params\\eqa_latlon.bin"
#define		SGTK_PS_LATLON_FILE_PATH	"..\\..\\params\\ps_latlon.bin"
#endif

#define		SGTK_CORRECT_SLOPE_ATTR_NAME		"Slope"
#define		SGTK_CORRECT_OFFSET_ATTR_NAME		"Offset"
#define		SGTK_CORRECT_SLOPE_REF_ATTR_NAME	"Slope_reflectance"
#define		SGTK_CORRECT_OFFSET_REF_ATTR_NAME	"Offset_reflectance"
#define		SGTK_CORRECT_BASE_ATTRIBUTE_NAME	"Base"

#define		SGTK_RANGE_MIN_ATTR_NAME		"Minimum_valid_DN"
#define		SGTK_RANGE_MAX_ATTR_NAME		"Maximum_valid_DN"
#define		SGTK_RANGE_ERROR_VALUE			"Error_DN"

// Config file object
class SGTK_ConfObj {
public:
	SGTK_ConfObj() {
		this->compress_type        = true;
		this->compress_level       = 4;
		this->chunk                = 256;
		this->byte_order           = true;						// true:LE  false:BE
		this->leap_sec_file_path   = SGTK_LEAP_SEC_FILE_PATH;
		this->eqr_latlon_file_path = SGTK_EQR_LATLON_FILE_PATH;
		this->eqa_latlon_file_path = SGTK_EQA_LATLON_FILE_PATH;
		this->ps_latlon_file_path  = SGTK_PS_LATLON_FILE_PATH;

		this->correct_slope_attr_name.push_back(SGTK_CORRECT_SLOPE_ATTR_NAME);
		this->correct_offset_attr_name.push_back(SGTK_CORRECT_OFFSET_ATTR_NAME);
		this->correct_slope_ref_attr_name.push_back(SGTK_CORRECT_SLOPE_REF_ATTR_NAME);
		this->correct_offset_ref_attr_name.push_back(SGTK_CORRECT_OFFSET_REF_ATTR_NAME);
		this->correct_base_attr_name.push_back(SGTK_CORRECT_BASE_ATTRIBUTE_NAME);
		this->range_min_attr_name.push_back(SGTK_RANGE_MIN_ATTR_NAME);
		this->range_max_attr_name.push_back(SGTK_RANGE_MAX_ATTR_NAME);
		this->range_err_value.push_back(SGTK_RANGE_ERROR_VALUE);
	};

	~SGTK_ConfObj(){};

	bool						compress_type;				    // DATATYPE_COMPRESS_TYPE
	int							compress_level;				    // DATATYPE_COMPRESS_LEVEL
	int							chunk;						    // DATATYPE_CHUNK
	bool						byte_order;					    // DATATYPE_BYTE_ORDER
	std::string					leap_sec_file_path;			    // LEAP_SECOND_FILE_PATH
	std::string					eqr_latlon_file_path;		    // EQR_LATLON_FILE_PATH
	std::string					eqa_latlon_file_path;		    // EQA_LATLON_FILE_PATH
	std::string					ps_latlon_file_path;		    // PS_LATLON_FILE_PATH
	std::vector<std::string>	correct_slope_attr_name;	    // CORRECT_SLOPE_ATTRIBUTE_NAME
	std::vector<std::string>	correct_offset_attr_name;	    // CORRECT_OFFSET_ATTRIBUTE_NAME
	std::vector<std::string>	correct_slope_ref_attr_name;	// CORRECT_SLOPE_REF_ATTRIBUTE_NAME
	std::vector<std::string>	correct_offset_ref_attr_name;	// CORRECT_OFFSET_REF_ATTRIBUTE_NAME
	std::vector<std::string>	correct_base_attr_name;		    // CORRECT_BASE_ATTRIBUTE_NAME
	std::vector<std::string>	range_min_attr_name;		    // RANGE_MIN_ATTRIBUTE_NAME
	std::vector<std::string>	range_max_attr_name;		    // RANGE_MAX_ATTRIBUTE_NAME
	std::vector<std::string>	range_err_value;			    // RANGE_ERROR_VALUE
};


// Byte order flag
class SGTK_ByteOrderObj {
public:
	SGTK_ByteOrderObj(){this->open_mode.clear();};
	~SGTK_ByteOrderObj(){};

	// set Byte order flag
	void	set_flg(hid_t id_file, bool mode){

		std::map<hid_t, bool>::iterator itr = this->open_mode.find(id_file);

		// key find result
		if (itr != this->open_mode.end()) {
			// this id_file handle open
			return;
		}
		else{
			// this id_file handle set
			this->open_mode[id_file] = mode;
		}
	}

	// get Byte order flag
	bool	get_flg(hid_t id_file){
		std::map<hid_t, bool>::iterator itr = this->open_mode.find(id_file);

		return itr->second;
	}

	// erase Byte order flag
	void	erase_flg(hid_t id_file) {
		std::map<hid_t, bool>::iterator itr = this->open_mode.find(id_file);

		if (itr != this->open_mode.end()) {
			this->open_mode.erase(itr);
		}
	}

private:
	std::map<hid_t, bool>	open_mode;		// true:LE  false:BE

};

#ifdef __SGTK_CNF_MAIN__
	SGTK_ConfObj		conf_obj;
	SGTK_ByteOrderObj	byte_order_obj;
#else
	extern SGTK_ConfObj			conf_obj;
	extern SGTK_ByteOrderObj	byte_order_obj;
#endif

int	load_config(const char* cnf_file_name);
int	find_attr_name_data(
						int							id_file,
						char*						label,
						std::vector<std::string>	label_items);

#endif
