/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2015 All rights reserved.                  */
/*  File Name    : SGTK_date_time.h                                          */
/*  Abstract     : I/O toolkit header file                                   */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2015/03/20                                                    */
/*  Version    1.00                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#ifndef SGTK_DATE_TIME_H
#define SGTK_DATE_TIME_H

#include <stdint.h>
static const double LEAPSEC_DEFAULT_TABLE[][3] = {
        { -15897599.000000, 48804.000000, 27.000000000000000 }, 
        {  15638401.000000, 49169.000000, 28.000000000000000 }, 
        {  47174402.000000, 49534.000000, 29.000000000000000 }, 
        {  94608003.000000, 50083.000000, 30.000000000000000 }, 
        { 141868804.000000, 50630.000000, 31.000000000000000 }, 
        { 189302405.000000, 51179.000000, 32.000000000000000 }, 
        { 410227206.000000, 53736.000000, 33.000000000000000 }, 
        { 504921607.000000, 54832.000000, 34.000000000000000 }, 
        { 615254408.000000, 56109.000000, 35.000000000000000 }
};
static const size_t LEAPSEC_DEFAULT_TABLE_SIZE = 
    sizeof(LEAPSEC_DEFAULT_TABLE) / sizeof(LEAPSEC_DEFAULT_TABLE[0]);

/*------------------------------------------------------------------*
 * Definition of ErrorCode.                                         *
 *------------------------------------------------------------------*/

/**
 * Main Error Code.
 * Error Codes are used in the main().
 * 
 * # Definition : AM2_ERR_<RETRY/NRETRY>_<KIND>
 * # Value      : 0x0000 ... 0x00FA [0...250]
 * 
 */
enum AM2_ERRORCODE_MAIN_COMMON
{
    /*--------------------------------------------------------------*
     * Normal : 0                                                   *
     *--------------------------------------------------------------*/
    AM2_NORMAL = 0,
    
    /*--------------------------------------------------------------*
     * Semi Normal : 1...50                                         *
     *--------------------------------------------------------------*/
    AM2_SEMI_NORMAL_ORBIT_ELMD = 2,
    AM2_SEMI_NORMAL_ORBIT_ELMP,
    AM2_SEMI_NORMAL_ORBIT_NOMINAL,
    AM2_SEMI_NORMAL_QA_FAIR,
    AM2_SEMI_NORMAL_QA_NG,
    
    /*--------------------------------------------------------------*
     * Error(Retry) : 51...150                                      *
     *--------------------------------------------------------------*/
    /* Input Error  : 111...120 */
    AM2_ERR_RTRY_IN_WO = 111,
    AM2_ERR_RTRY_IN_COMPARA,
    AM2_ERR_RTRY_IN_ASD,
    AM2_ERR_RTRY_IN_L1AB_PRODUCT,
    AM2_ERR_RTRY_IN_L1R_PRODUCT,
    AM2_ERR_RTRY_IN_PRODUCT,
    AM2_ERR_RTRY_IN_L2_PRODUCT,
    AM2_ERR_RTRY_IN_L3_PRODUCT,
    
    /* Outnput Error  : 121...130 */
    AM2_ERR_RTRY_OUT_LOG = 121,
    AM2_ERR_RTRY_OUT_L1AB_PRODUCT,
    AM2_ERR_RTRY_OUT_L1R_PRODUCT,
    AM2_ERR_RTRY_OUT_L2_PRODUCT,
    AM2_ERR_RTRY_OUT_L3_PRODUCT,
    AM2_ERR_RTRY_OUT_RESULTFILE,
    AM2_ERR_RTRY_OUT_WORKREPORT,
    
    /* Internal I/F Error  : 131...140 */
    AM2_ERR_RTRY_INTER_FILE = 131,
    
    /* Other Error  : 141...150  */
    AM2_ERR_RTRY_OTHER_SYSTEM = 141,
    
    /*--------------------------------------------------------------*
     * Error(No Retry) : 151...250                                  *
     *--------------------------------------------------------------*/
    /* Internal I/F Error  : 211...220 */
    AM2_ERR_NRTRY_INTER_LOGIC = 211,
    
    /* Other : 221...250 */
    AM2_ERR_NRTRY_LAYTENCY_NG = 221,
    AM2_ERR_NRTRY_SCAN_NUMBER_NG,
};


/**
 * Internal Error Code.
 * Error Codes are used in the Sub-Functions.
 * 
 * # Definition : <KIND>_ERROR_<DETAIL>
 * # Value      : 0x0000 ... 
 * 
 */
enum AM2_ERRORCODE_SUB
{
    /* No Error */
    NORMAL = 0x0000,
    
    FREAD_ERROR_SEA_FLG,
    MEM_ERROR_SEA_FLG,
    MEM_ERROR_BROWS_THUMNAIL,
    OPENDIR_ERROR_BROWS_THUMBNAIL,
    ARGUMENT_ERROR,
    WO_ERROR_STRUCTURE_LEVEL,
    WO_PARA_FILE_OPEN_ERROR,
    ILLEGAL_LEVEL_ERROR,
    ILLEGAL_IO_FLAG_ERROR,
    WO_FILE_OPEN_ERROR,
    
    WR_STRUCTURE_LEVEL_ERROR,
    WR_PARA_FILE_OPEN_ERROR,
    WR_FILE_OPEN_ERROR,
    NORMAL_NO_OVERLAP_SCAN,
    ILLEGAL_ERROR_PASS_NUM,
    ILLEGAL_ERROR_ORBIT_DIRECTION,
    TIME_ERROR_UTC,
    ILLEGAL_ERROR_PROJECTION,
    NO_NODE,
    ZERO_DIVIDE_ERROR,
    
    COUNT_VALUE_ERROR,
    PARAMETER_ERROR,
    NO_EXIT_SEA_FLG_DATABASE_FILE,
    FOPEN_ERROR_PARAM,
    FREAD_ERROR_PARAM,
    FOPEN_ERROR_SCENE_INFO,
    MEM_ERROR_ALLOCATE,
    FOPEN_ERROR_ORBIT_INTERNAL,
    FOPEN_ERROR_ATTITUDE_INTERNAL,
    FWRITE_ERROR_ORBIT_INTERNAL,
    
    FWRITE_ERROR_ATTITUDE_INTERNAL,
    FOPEN_ERROR_EQUATION_TIME_DATA,
    FREAD_ERROR_EQUATION_TIME_DATA,
    FOPEN_ERROR_PROCESS_RESULT,
    FWRITE_ERROR_PROCESS_RESULT,
    ILLEGAL_COM_DIR,
    ILLEGAL_VERSION,
    ILLEGAL_TIME,
    ILLEGAL_FILE_NAME,
    ILLEGAL_PROCESS_LEVEL,
    
    FOPEN_ERRORASD_FILE,
    FOPEN_ERROR_LEAPSECOND,
    MEM_ERROR_LEAPSECOND,
    NO_EXIST_ERROR_LEAPSECOND_TABLE,
    TIME_ERROR_LEAPSECOND_TABLE,
    TIME_ERROR_INPUT_TIME,
    FOPEN_ERROR_PROCESS_DONE,
    FREAD_ERROR_PROCESS_DONE,
    FWRITE_ERROR_PROCESS_DONE,
    NO_EXIST_ERROR_PACKET_1,
    
    XML_ERROR_NONE_TAGPATH,
    XML_ERROR_NEW_DOC,
    XML_ERROR_PARSE_FILE,
    XML_ERROR_XPATH_NEW_CONTEXT,
    XML_ERROR_XPATH_EVAL_EXPRESSION,
    XML_ERROR_XPATH_NODESETVAL,
    XML_ERROR_SAVE_FORMAT_FILE_ENC,
    XML_ERROR_CHK_SCHEMA,
    XML_ERROR_CHK_FILE,
    PARAM_ERROR_NULLPOINTER,
    
    PARAM_ERROR_BUFFERSIZE,
    NO_EXIST_ERROR_RECORD_NUM,
    NO_EXIST_ERROR_START_TIME,
    FOPEN_ERROR_RSMP_COEF,
    FREAD_ERROR_RSMP_COEF,
    ALL_DATA_ERROR,
    ILLEGAL_ERROR_PACKET_NO,
    FREAD_ERROR_ORBIT_INTERNAL,
    FREAD_ERROR_ATTITUDE_INTERNAL,
    DSEF_ERROR_NOT_INTERPOLATE,
    
    FOPEN_ERROR_DSEF,
    FACCESS_ERROR_DSEF,
    MEM_ERROR_DSEF,
    FFORMAT_ERROR_DSEF,
    DSEF_ERROR_NONPERIODIC,
    DSEF_ERROR_INPUT_TIME,
    FOPEN_ERROR_MD_META,
    FREAD_ERROR_MD_META,
    NO_EXIST_ERROR_PROCESS_KIND,
    LOG_FILE_OPEN_ERROR,
    
    ORBIT_TYPE_ERROR,
    CALC_ERROR_DIVISION_0,
    MEM_RELEASE_ERROR,
    ILLEGAL_FREAQUENCY_ERROR,
    ILLEGAL_POLARIZATION_ERROR,
    ILLIGAL_SYSTEM_LOGIC_ERROR,
    ILLEGAL_PURODUCT_FILE_ERROR,
    ERORR_VALUE_INPUT,
    NOMINAL_FILE_OPEN_ERROR,
    MEM_ERROR_PRODUCT,
    
    FOPEN_ERROR_PRODUCTFILE,
    FCLOSE_ERROR_PRODUCTFILE,
    NO_EXIST_ERROR_META_DATA,
    NO_EXIST_ERROR_PRODUCT_DATA,
    NO_EXIST_ERROR_DIMENSION_SIZE,
    NO_EXIST_ERROR_ACCESS_LABEL,
    ILLEGAL_ERROR_DIMENSION_SIZE,
    ILLEGAL_ERROR_SCAN_NUM,
    ILLEGAL_ERROR_PRODUCT_NAME,
    ILLEGAL_ERROR_DIR_PATH,
    
    ILLEGAL_ERROR_GEOPHYSICAL_NUM,
    ILLEGAL_ERROR_RESOLUTION,
    ILLEGAL_ERROR_BRIGHTNESS_OR_GEOPHYSICAL,
    IRRIGAL_UTC_ERROR,
    FOPEN_ERROR_PATH_LONGITUDE,
    FREAD_ERROR_PATH_LONGITUDE,
    DATASIZE_OVER_ERROR,
    IMAGE_CONFIG_FILE_OPEN_ERROR,
    NO_EXIST_ERROR_TAG_NAME,
    ILLEGAL_SENSOR_ERROR,
    
    ILLEGAL_SCAN_NUM,
    FOPEN_ERROR_IN_FILE,
    FOPEN_ERROR_OUT_FILE,
    FREAD_ERROR_IN_FILE,
    FWRITE_ERROR_OUT_FILE,
    COM_ERROR_COPY_FILE
};

/*------------------------------------------------------------------*
 * Definition of Constant.                                          *
 *------------------------------------------------------------------*/

/**
 * Logout Verbose Mode File [GCOM-PR-100043].
 */
#define LOGOUT_VERBOSE_MODE_FILE   ("log_verbosemode_on")

/** Number of Frequency. */
enum
{
        FREQUENCY_06,
        FREQUENCY_07,
        FREQUENCY_10,
        FREQUENCY_18,
        FREQUENCY_23,
        FREQUENCY_36,
        FREQUENCY_89A,
        FREQUENCY_89B,
        FREQUENCY_NUM
};

/** Number of Polarization (V),(H). */
enum
{
        POLARIZATION_V,
        POLARIZATION_H,
        POLARIZATION_NUM
};

/** orbit event access label(use this label when you 'use com_get_orbit_info' function) */
enum OrbitEventAccessLavel
{
    ASCENDPT_TIME,
    DESCENDPT_TIME,
    NORTHPT_TIME,
    SOUTHPT_TIME
};

/** Number of Sensor. (Engineering value conversion Sensor.) */
enum
{
    THERMISTOR1,
    THERMISTOR2,
    THERMISTOR3,
    PLATINUM1,
    PLATINUM2,
    PLATINUM3
};
    

#define ALL_FREQUENCY FREQUENCY_NUM

/*------------------------------------------------------------------*/

#define QA_FLAG_SIZE                                                  513
#define FOOT_INFO_NO_1                                                0
#define FOOT_INFO_NO_6                                                5
#define FOOT_INFO_NO_7                                                6
#define FOOT_INFO_NO_12                                               11
#define FOOT_INFO_NO_13                                               12
#define FOOT_INFO_NO_15                                               14
#define FOOT_INFO_NO_16                                               15
#define FOOT_INFO_NO_21                                               20
#define FOOT_INFO_NO_22                                               21
#define FOOT_INFO_NO_24                                               23
#define FOOT_INFO_NO_25                                               24
#define FOOT_INFO_NO_30                                               29
#define FOOT_INFO_NO_31                                               30
#define FOOT_INFO_NO_36                                               35
#define FOOT_INFO_NO_37                                               36
#define FOOT_INFO_NO_42                                               41
#define FOOT_INFO_NO_43                                               42
#define FOOT_INFO_NO_45                                               44
#define FOOT_INFO_NO_46                                               45
#define FOOT_INFO_NO_47                                               46
#define FOOT_INFO_NO_48                                               47
#define FOOT_INFO_NO_51                                               50
#define FOOT_INFO_NO_52                                               51
#define FOOT_INFO_NO_53                                               52
#define FOOT_INFO_NO_54                                               53
#define FOOT_INFO_NO_55                                               54
#define FOOT_INFO_NO_60                                               59
#define AVAILABLE                                                     "Available"
#define NOT_AVAILABLE                                                 "Not Available"
#define CODE_00                                                       "00"
#define CODE_01                                                       "01"
#define CODE_02                                                       "02"
#define CODE_04                                                       "04"
#define CODE_33                                                       "33"
#define CODE_NORMAL                                                   CODE_00
#define TIME_OF_QUARTER                                             (98.9*60.0/4.0)
#define PATH_NO_INTERVAL                                                16
#define MAX_PATH_NO                                                     233

#define PCD_NAVI_GPS_MONTH_POS      7
#define PCD_NAVI_GPS_SECOND_POS     9
#define PCD_NAVI_NAVI_TIME_POS      12
#define PCD_NAVI_NAVI_POS_X_POS     16
#define PCD_NAVI_NAVI_POS_Y_POS     19
#define PCD_NAVI_NAVI_POS_Z_POS     22
#define PCD_NAVI_NAVI_SPEED_X_POS   25
#define PCD_NAVI_NAVI_SPEED_Y_POS   28
#define PCD_NAVI_NAVI_SPEED_Z_POS   31
#define PCD_NAVI_LAT_ARG_POS        52
#define PCD_NAVI_ATT_TIME_POS       60
#define PCD_NAVI_ATT_MARGIN_R_POS   34
#define PCD_NAVI_ATT_MARGIN_P_POS   37
#define PCD_NAVI_ATT_MARGIN_Y_POS   40
#define PCD_NAVI_ATT_RATE_R_POS     43
#define PCD_NAVI_ATT_RATE_P_POS     46
#define PCD_NAVI_ATT_RATE_Y_POS     49
#define PCD_NAVI_ATT_FLAG_POS       61
#define PCD_NAVI_ORBIT_STS_POS      56

#define S_H_GPS_WEEK_POS        9
#define S_H_GPS_SEC_POS         11
#define S_H_FINE_TIME_POS       14

#define SUPPLEMENT_BLK_1_POS    80
#define SUPPLEMENT_BLK_1_SIZE   32
#define SUPPLEMENT_BLK_2_POS    180
#define SUPPLEMENT_BLK_2_SIZE   40
#define SUPPLEMENT_BLK_3_POS    256
#define SUPPLEMENT_BLK_3_SIZE   48
#define SUPPLEMENT_BLK_4_POS    512
#define SUPPLEMENT_BLK_4_SIZE   24
#define SUPPLEMENT_BLK_5_POS    692
#define SUPPLEMENT_BLK_5_SIZE   104
#define SUPPLEMENT_TACHO_1_POS  0
#define SUPPLEMENT_TACHO_5_POS  8

#define PROCESS_KIND_SG         "SG"
#define PROCESS_KIND_SN         "SN"
#define PROCESS_KIND_SL         "SL"
#define INFILE_NAME_SIZE        30000
#define FILE_NAME_SIZE          129
#define FOOT_INFO_NUM           60
#define FOOT_INTERVAL_PIXEL     5.0
#define FOOT_INTERVAL_PIXEL_H   97
#define FOOT_INTERVAL_PIXEL_L_1 49
#define FOOT_INTERVAL_PIXEL_L_2 48
#define FOOT_START_SCAN_NUM     6
#define SCAN_INTERVAL_1         15
#define POINT_NUM_1             6
#define SCAN_INTERVAL_2         100
#define POINT_NUM_2             3
#define SCAN_INTERVAL_3         150
#define POINT_NUM_3             3
#define FOOT_LOSS_VALUE         "********"
#define REQUIRED_SCAN_NUM       1690
#define FOOT_INFO_NO_1          0
#define FOOT_INFO_NO_6          5
#define FOOT_INFO_NO_7          6
#define G_ID_PROCESS_LVL        25
#define G_ID_PROCESS_KIND       27
#define G_ID_RESOLUTION         32
#define PROCESS_LVL_L1          "L1"
#define PROCESS_LVL_L2          "L2"
#define PROCESS_RESULT_FILE_EXTENSION   ".xml"


#define PROCESS_LEVEL_L1A       0
#define PROCESS_LEVEL_L1B       1
#define PROCESS_LEVEL_L1R       2
#define PROCESS_LEVEL_L2        3
#define PROCESS_LEVEL_L3D       4
#define PROCESS_LEVEL_L3M       5

#define PACKET_HEADER_SIZE      16
#define SEQ_COUNT_NUM           0x4000

#define CONV_ATT                0.001
#define CONV_ATT_V              0.0001
#define CONV_NAVI_V             0.001
#define CONV_LAT_ARG            0.00002
#define CONV_MS_SEC             0.001
#define CONV_SEC_MS             1000                

#define LOSS_VALUE_NAVI         -9999999
#define LOSS_VALUE_NAVI_V       -9999.999
#define LOSS_VALUE_ATT          -9999.999
#define LOSS_VALUE_ATT_V        -999.9999
#define LOSS_VALUE_LAT_ARG      -99999.99999
#define LOSS_VALUE_TIME         0.0
#define LOSS_VALUE_ORBIT_STS    -1
#define LOSS_VALUE_ATT_FLAG     -1

#define TAG_FILE_NAME_COM_PARA_IN_SCHEMA    "/DataProcessingDeviceParaFileInfo/SchemaDirectoryInfo/ComParaIn"
#define TAG_FILE_NAME_L1_PROCRSLT_SCHEMA    "/DataProcessingDeviceParaFileInfo/SchemaDirectoryInfo/L1ProcessResult"
#define TAG_FILE_NAME_L2_PROCRSLT_SCHEMA    "/DataProcessingDeviceParaFileInfo/SchemaDirectoryInfo/L2ProcessResult"
#define TAG_FILE_NAME_L3_PROCRSLT_SCHEMA    "/DataProcessingDeviceParaFileInfo/SchemaDirectoryInfo/L3ProcessResult"
#define TAG_FILE_NAME_LEAP_SECOND       "/Level1/ParameterFileName/leap_second_filename"
#define TAG_FILE_NAME_LEAP_SECOND_L2    "/Level2/ParameterFileName/leap_second_filename"
#define TAG_FILE_NAME_LEAP_SECOND_L3    "/Level3/ParameterFileName/leap_second_filename"
#define TAG_FILE_NAME_COM_PARA_IN_L2    "/Level2/ParameterFileName/com_para_in_FileName"
#define TAG_FILE_NAME_COM_PARA_IN_L3    "/Level3/ParameterFileName/com_para_in_FileName"
#define TAG_FILE_NAME_CORE_META_L1      "/Level1/ParameterFileName/CoreMetaDataOriginal"
#define TAG_FILE_NAME_CORE_META_L2      "/Level2/ParameterFileName/CoreMetaDataOriginal"
#define TAG_FILE_NAME_CORE_META_L3      "/Level3/ParameterFileName/CoreMetaDataOriginal"
#define TAG_SCENE_START_TIME            "/SceneInformation/SceneStartTime"
#define TAG_SCENE_END_TIME              "/SceneInformation/SceneEndTime"

/*------------------------------------------------------------------*/

#define LOW_PIXEL_NUM_CSM_HTS   16
#define HIGH_PIXEL_NUM_CSM_HTS  32

#define GOOD_CHAR               "Good"
#define POOR_CHAR               "Poor"
#define OK_CHAR                 "OK"
#define NG_CHAR                 "NG"
#define FAIR_CHAR               "Fair"
#define CHK_OK                  0
#define CHK_NG                  1

#define PACKET_NO_1             0

#define TO_AND_FRO_SCAN_NUM     2
#define NO_CHK_DATA             -1
#define ALL_LIMIT_ERROR         1
#define NUM_TACHO_CNT           5
#define NO_EXIST_SCANTIME       -1.0
#define SPC_SPS_ERROR_FLG_NUM   4
#define SPC_SPS_OK              0
#define SPC_NG                  1
#define SPS_NG                  2
#define SPC_SPS_NG              3
//#define   ATTITUDE_DATA_NUM       6
#define ATTITUDE_ELEMENT_NUM    3

#define DSEF_EXTRAPOLATE        1
#define ONBOARD_CHAR            "ONBOARD"
#define ELMD_CHAR               "ELMD"
#define ELMP_CHAR               "ELMP"
#define NOMINAL_CHAR            "NOMINAL"
//#define   NAVIGATION_DATA_NUM     6

//#define   DOT_NUM                 4
#define NO_EXIST                -1
#define INTERPOLATION_ON        1
#define INTERPOLATION_OFF       0

#define CANNOT_INTERPOLATE      9
#define NAVIGATION_ELEMENT_NUM  3

#define FRQ_L_NUM               6
#define FRQ_H_NUM               2

#define SCANDQ_SCAN_QUALITY_FLG_OBS_START_TIME_CHECK                    0
#define SCANDQ_SCAN_QUALITY_FLG_ANTENNA_DIFFERENCE_COUNT_CHECK          1
#define SCANDQ_SCAN_QUALITY_FLG_ANTENNA_ROTATION_CHECK                  2
#define SCANDQ_SCAN_QUALITY_FLG_HTS_TEMPERATURE_CHECK                   3
#define SCANDQ_SCAN_QUALITY_FLG_ADA_STATOR_CHECK                        4
#define SCANDQ_SCAN_QUALITY_FLG_ADA_ROTOR_A_CHECK                       5
#define SCANDQ_SCAN_QUALITY_FLG_ADA_ROTOR_B_CHECK                       6
#define SCANDQ_SCAN_QUALITY_FLG_ADA_ANGULAR_MOMENTUM_CHECK              7
#define SCANDQ_SCAN_QUALITY_FLG_HTS_VALUE_LIMIT_CHECK                   8
#define SCANDQ_SCAN_QUALITY_FLG_CSM_VALUE_LIMIT_CHECK                   9
#define SCANDQ_SCAN_QUALITY_FLG_HTS_DIFFERENCE_LIMIT_CHECK              10
#define SCANDQ_SCAN_QUALITY_FLG_CSM_DIFFERENCE_LIMIT_CHECK              11
#define SCANDQ_SCAN_QUALITY_FLG_HTS_HTS_CONTROL_TEMPERATURE_LIMIT_CHECK 12
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_13                             13      // nodefined Scan Data Quality flag 14bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_14                             14      // nodefined Scan Data Quality flag 15bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_15                             15      // nodefined Scan Data Quality flag 16bit
#define SCANDQ_SCAN_QUALITY_FLG_ATTITUDE_FLUG                           16      //Scan Data Quality flag 17bit
#define SCANDQ_SCAN_QUALITY_FLG_ATTITUDE_ERROR_ANGLE_CHECK              17      //Scan Data Quality flag 18bit
#define SCANDQ_SCAN_QUALITY_FLG_ATTITUDE_ANGULAR_VELOCITY_CHECK         18      //Scan Data Quality flag 19bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_19                             19      // nodefined Scan Data Quality flag 20bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_20                             20      // nodefined Scan Data Quality flag 21bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_21                             21      // nodefined Scan Data Quality flag 22bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_22                             22      // nodefined Scan Data Quality flag 23bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_23                             23      // nodefined Scan Data Quality flag 24bit
#define SCANDQ_SCAN_QUALITY_FLG_NAVIGATION_FLUG_LSB                     24      //Scan Data Quality flag 25bit
#define SCANDQ_SCAN_QUALITY_FLG_NAVIGATION_FLUG_MSB                     25      //Scan Data Quality flag 26bit
#define SCANDQ_SCAN_QUALITY_FLG_ALTITUDE_CHECK                          26      //Scan Data Quality flag 27bit
#define SCANDQ_SCAN_QUALITY_FLG_SATELLITE_VELOCITY_CHECK                27      //Scan Data Quality flag 28bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_28                             28      // nodefined Scan Data Quality flag 29bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_29                             29      // nodefined Scan Data Quality flag 30bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_30                             30      // nodefined Scan Data Quality flag 31bit
#define SCANDQ_SCAN_QUALITY_FLG_NODEFINE_31                             31      // nodefined Scan Data Quality flag 32bit

#define HTS_TEMP_NUM            10
#define HTS_TEMP_CONTRL_NUM     5

#define NAVI_FLAG_KALMAN_FILTER 2
#define NAVIGATION_STOP 0

#define FLG_CONTAIN_MISS_DATA       1
#define FLG_NO_CONTAIN_MISS_DATA    0

//#define   PLATINUM1_LEVEL_NUM         3
//#define   PLATINUM2_LEVEL_NUM         3
//#define   PLATINUM3_LEVEL_NUM         10
//#define   THERMISTOR1_LEVEL_NUM       4
//#define   THERMISTOR2_LEVEL_NUM       3
//#define   THERMISTOR3_LEVEL_NUM       3
#define THERMISTOR1_LAYER_NUM       4
#define THERMISTOR2_LAYER_NUM       3
#define THERMISTOR3_LAYER_NUM       3
#define PLATINUM1_LAYER_NUM         3
#define PLATINUM2_LAYER_NUM         3
#define PLATINUM3_LAYER_NUM         10
#define PLATINUM4_LAYER_NUM         3


#define HTS_TEMP_POS                6
#define HTS_TEMP_CONTRL_POS         0

#define OBS_SUPPLY_TACHO_CNT_POS    0
#define SPC_TEMP_ADA_STATOR_POS     25
#define SPS_TEMP_ADA_ROTOR_A_POS    41
#define SPS_TEMP_ADA_ROTOR_B_POS    42
#define OBS_SUPPLY_ADA_MOV_POS      72
#define SPC_ERROR_FLG_POS   16
#define SPS_ERROR_FLG_POS   136
#define PCD_DATA_NAVIGATION_FILENAME    "navi_data.csv"
#define PCD_DATA_ATTITUDE_FILENAME  "att_data.csv"

#define LIMIT_NUM   2
#define MEM_ALLOCATE_ERROR                  12 
#define NOMINAL_ATTITUDE_TOLERANCE_ROLL     0.0
#define NOMINAL_ATTITUDE_TOLERANCE_PITCH    0.0
#define NOMINAL_ATTITUDE_TOLERANCE_YAW      0.0
#define TACHO_PARAM (1.0/pow(2.0,14)*60.0/(187.5*pow(10.0,-3))/4.0);
#define ATTITUDE_FLG_EA_IRU_BASE    0
#define INTERPOLATED_FLG    2
#define TIME_DIFF_BACKWARD      4.0

/*------------------------------------------------------------------*
 * position in orbit                                                *
 *------------------------------------------------------------------*/
#define NOMINAL_ORBIT                       99

/*------------------------------------------------------------------*/

#define     CHK_OK                                0
#define     CHK_NG                                1

#define     FRQ_L_NUM                             6
#define     FRQ_H_NUM                             2
#define     LOW_PIXEL_NUM                       243
#define     HIGH_PIXEL_NUM                      486
#define     HTS_TEMP_NUM                         10
#define     HIGH_PIXEL_CENTER                   244
#define     NAVIGATION_DATA_NUM                   6
#define     ATTITUDE_DATA_NUM                     6
#define     LOW_PIXEL_NUM_CSM_HTS                16
#define     HIGH_PIXEL_NUM_CSM_HTS               32
#define     SUPPLEMENT_SIZE                     248
#define     SPC_COUNT_NUM                        34
#define     SPS_COUNT_NUM                        46
#define     SEA_TABLE_LAT_NUM                   100
#define     SEA_TABLE_LON_NUM                   100
#define     PCD_DATA_SIZE                        64
#define     FILENAME_LENGTH                     512

#define     NEARLY_ZERO                           0.000000001
#define     ANTENNA_ROTATION                      1.5


#define     GRING_POINT_NUM                       8
#define     BUFF_SIZE_16                         16
#define     BUFF_SIZE_32                         32
#define     BUFF_SIZE_128                       128
#define     RESOLUTION_POS                       32
#define     PRODUCT_ID_POS                       29
#define     PROJECTION_POS_L3                    20
#define     PRODUCT_STATISTICAL_PERIOD_POS       16

#define     TBD_CHAR                            "TBD"
#define     GOOD_CHAR                           "Good"
#define     PASS_CHAR                           "1"
#define     FAIL_CHAR                           "0"
#define     HIGH_CHAR                           "High"
#define     LOW_CHAR                            "Low"

#define     SPATIAL_REPRESENTATION_TYPE_L3      "002"
#define     MEASURE_VAL_0_25                    "0.25"
#define     MEASURE_VAL_25                      "25"
#define     MEASURE_VAL_0_1                     "0.1"
#define     MEASURE_VAL_10                      "10"
#define     PROJECTION_EQR                      "EQR"
#define     PROJECTION_PSN                      "PS-N"
#define     GEOPHY_NAME_SND                     "Snow Depth"
#define     MEASURE_TYPE_DEGREE                 "degree"
#define     MEASURE_TYPE_KM                     "km"
#define     PID_TB_06G                          "T06"
#define     PID_TB_07G                          "T07"
#define     PID_TB_10G                          "T10"
#define     PID_TB_18G                          "T18"
#define     PID_TB_23G                          "T23"
#define     PID_TB_36G                          "T36"
#define     PID_TB_89G                          "T89"
#define     PID_SND                             "SND"

#define     ARGUMENT_NUM                          6
#define     WO_FILE_PATH                          1
#define     DEVICE_ID                             2
#define     CONFIGURATION_FILE_PATH               3
#define     PROCESS_ID                            4
#define     SUPPORT_FILE                          5

#define     DEFAULT_ARRAY_NO                      0

#define     IO_FLAG_INPUT                         0
#define     IO_FLAG_OUTPUT                        1

#define     SEA_FLAG_FILE_NAME_LENGTH           128
#define     DIRECTORY_LENGTH                   1024
#ifndef UNIT_TEST_AM2_SEA_FLG_MAIN
#define     MAX_SEA_FLAG_SIZE_LAT             20005
#define     MAX_SEA_FLAG_SIZE_LON              5000
#define     MAX_ALTITUDE_SIZE_LAT              9000
#define     MAX_ALTITUDE_SIZE_LON             18000
#else

#define     MAX_SEA_FLAG_SIZE_LAT              1800
#define     MAX_SEA_FLAG_SIZE_LON               180
#define     MAX_ALTITUDE_SIZE_LAT               900
#define     MAX_ALTITUDE_SIZE_LON              1800
#endif

#define     SEA_FLG_LAT_START_VALUE             -90
#define     SEA_FLG_LAT_END_VALUE               90
#define     SEA_FLG_LON_START_VALUE             -180
#define     SEA_FLG_LON_END_VALUE               180
#define     SEA_FLG_LAT_INTERVAL                0.02
#define     SEA_FLG_LON_INTERVAL                0.02

#define     SEA_FLG_LAT_CHK_OK                  0
#define     SEA_FLG_LAT_CHK_NG                  1
#define     SEA_FLG_LON_CHK_OK                  0
#define     SEA_FLG_LON_CHK_NG                  1

#define     ALTITUDE_LAT_START_VALUE            -90
#define     ALTITUDE_LAT_END_VALUE              90
#define     ALTITUDE_LON_START_VALUE            -180
#define     ALTITUDE_LON_END_VALUE              180
#define     ALTITUDE_LAT_INTERVAL               0.02
#define     ALTITUDE_LON_INTERVAL               0.02

#define     ALTITUDE_LAT_CHK_OK                 0
#define     ALTITUDE_LAT_CHK_NG                 1
#define     ALTITUDE_LON_CHK_OK                 0
#define     ALTITUDE_LON_CHK_NG                 1

#define     LEVEL_1A_CHAR                       "L1A"
#define     LEVEL_1B_CHAR                       "L1B"
#define     LEVEL_1R_CHAR                       "L1R"
#define     LEVEL_2_CHAR                        "L2"
#define     LEVEL_3_DAY_CHAR                    "L3D"
#define     LEVEL_3_MONTH_CHAR                  "L3M"

#define     FREQUENCY_6_CHAR                    "6GHz"
#define     FREQUENCY_7_CHAR                    "7GHz"
#define     FREQUENCY_10_CHAR                   "10GHz"
#define     FREQUENCY_18_CHAR                   "18GHz"
#define     FREQUENCY_23_CHAR                   "23GHz"
#define     FREQUENCY_36_CHAR                   "36GHz"
#define     FREQUENCY_89A_CHAR                  "89AGHz"
#define     FREQUENCY_89B_CHAR                  "89BGHz"
#define     FREQUENCY_89_CHAR                   "89GHz"

#define     TYPE_89A_CAHR                       "89A"
#define     TYPE_89B_CAHR                       "89B"

#define     POLARIZATION_V_CHAR                 "V"
#define     POLARIZATION_H_CHAR                 "H"

#define     CREATION_YES_CHAR                   "YES"



typedef struct {
    double          x;
    double          y;
    double          z;
} vector3;

typedef struct {
    double  re;
    double  inv_f;
    vector3 d;
} ellip;

//#define       RSMP_COEF_DIRECTORY                 "."

#define     SCHEMA_FILENAME_L0_WORK_ORDER       "./schema_l0.xml"
#define     SCHEMA_FILENAME_L1_WORK_ORDER       "./schema_l1.xml"
#define     SCHEMA_FILENAME_L2_WORK_ORDER       "./schema_l2.xml"
#define     SCHEMA_FILENAME_L3_WORK_ORDER       "./schema_l3.xml"
#define     SCHEMA_FILENAME_CPARA_EX            "./schema_ex.xml"
#define     SCHEMA_FILENAME_CPARA_IN            "./schema_in.xml"
#define     SCHEMA_FILENAME_CPARA_EX_PERIOD     "./schema_ex_period.xml"

#define     MODULE_INPUT                        "InputModule"
#define     MODULE_PRE_CHECK                    "PreprocessModule"
#define     MODULE_RADIOMETRIC                  "RadiometricModule"
#define     MODULE_GEOMETRIC                    "GeometricModule"
#define     MODULE_RESAMPLING                   "ResumplingModule"
#define     MODULE_SCENE_EXTRACT                "SceneExtractModule"
#define     MODULE_CALC_BT                      "BTcalculateModule"
#define     MODULE_CUT_OVERLAP_HIGH             "CutOverlapHigh"
#define     MODULE_CUT_OVERLAP_LOW              "CutOverlapLow"
#define     MODULE_WORK_REPORT                  "WorkReport"
#define     MODULE_SEA_FLAG_L1A                 "SeaFlagL1aModule"
#define     MODULE_SEA_FLAG_L1R                 "SeaFlagL1rModule"
#define     MODULE_BROWS_THUMNAIL               "BrowsThumbnailModule"
#define     MODULE_L1_BROWS_THUMNAIL            "BrowsThumbnailL1Module"
#define     MODULE_L2_BROWS_THUMNAIL            "BrowsThumbnailL2Module"
#define     MODULE_L3_BROWS_THUMNAIL            "BrowsThumbnailL3Module"
#define     MODULE_L1AB_PRODUCT                 "ProductOutputL1abModule"
#define     MODULE_L1R_PRODUCT                  "ProductOutputL1rModule"
#define     MODULE_L2_PRODUCT                   "ProductOutputL2Module"
#define     MODULE_L3_PRODUCT                   "ProductOutputL3Module"
#define     MODULE_PROCESS_RESULT               "ProcessResultModule"


#define     TAG_RESULT_ROOT                             "/ProcessResultFile"
#define     TAG_FILE_IDENTIFIER                         "/ProcessResultFile/MD_Metadata/fileIdentifier"
#define     TAG_DATE_STAMP                              "/ProcessResultFile/MD_Metadata/dateStamp"
#define     TAG_DATE_1                                  "/ProcessResultFile/MD_Metadata/identificationInfo/MD_Identification/citation/CI_Citation/date/CI_Date/date"
#define     TAG_BROWSE_FILE                             "/ProcessResultFile/MD_Metadata/identificationInfo/MD_Identification/graphicOverview/MD_BrowseGraphic/fileName"
#define     TAG_THUMNAIL_FILE                           "/ProcessResultFile/MD_Metadata/identificationInfo/MD_Identification/graphicOverview/MD_ThumbnailGraphic/fileName"
#define     TAG_SPATIAL_REPRESENTATION_TYPE             "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/spatialRepresentationType"
#define     TAG_MEASURE_VALUE                           "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/spatialResolution/distance/Measure/value"
#define     TAG_MEASURE_TYPE                            "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/spatialResolution/distance/Measure/uom/UnitOfMeasure/measurementType"
#define     TAG_WEST_BOUND                              "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/extent/EX_Extent/GeographicElement/EX_GeographicBoundingBox/westBoundLongitude"
#define     TAG_EAST_BOUND                              "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/extent/EX_Extent/GeographicElement/EX_GeographicBoundingBox/eastBoundLongitude"
#define     TAG_SOUTH_BOUND                             "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/extent/EX_Extent/GeographicElement/EX_GeographicBoundingBox/southBoundLatitude"
#define     TAG_NORTH_BOUND                             "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/extent/EX_Extent/GeographicElement/EX_GeographicBoundingBox/northBoundLatitude"
#define     TAG_OBS_START_DATE_TIME                     "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/extent/EX_Extent/temporalElement/EX_TemporalExtent/extent/TM_Primitive/beginEnd/startEndTime/begin"
#define     TAG_OBS_END_DATE_TIME                       "/ProcessResultFile/MD_Metadata/identificationInfo/MD_DataIdentification/extent/EX_Extent/temporalElement/EX_TemporalExtent/extent/TM_Primitive/beginEnd/startEndTime/end"
#define     TAG_DATE_2                                  "/ProcessResultFile/MD_Metadata/dataQualityInfo/DQ_DataQuality/report/DQ_Element/result/DQ_ConformanceResult/specification/CI_Citation/date/CI_Date/date"
#define     TAG_PASS                                    "/ProcessResultFile/MD_Metadata/dataQualityInfo/DQ_DataQuality/report/DQ_Element/result/DQ_ConformanceResult/pass"
#define     TAG_PRDCT_PRODUCT_NAME                      "/ProcessResultFile/ProductMetaData/ProductName"
#define     TAG_PRDCT_GEOPHYSICAL_NAME                  "/ProcessResultFile/ProductMetaData/GeophysicalName"
#define     TAG_PRDCT_PRODUCT_VERSION                   "/ProcessResultFile/ProductMetaData/ProductVersion"
#define     TAG_PRDCT_ALGORITHM_VERSION                 "/ProcessResultFile/ProductMetaData/AlgorithmVersion"
#define     TAG_PRDCT_PARAMETER_VERSION                 "/ProcessResultFile/ProductMetaData/ParameterVersion"
#define     TAG_PRDCT_PRODUCT_SIZE_MBYTE                "/ProcessResultFile/ProductMetaData/ProductSize_MByte"
#define     TAG_PRDCT_GRANULE_ID                        "/ProcessResultFile/ProductMetaData/GranuleID"
#define     TAG_PRDCT_OPERATION                         "/ProcessResultFile/ProductMetaData/Operation"
#define     TAG_PRDCT_PRODUCTION_DATE_TIME              "/ProcessResultFile/ProductMetaData/ProductionDateTime"
#define     TAG_PRDCT_OBS_START_DATE_TIME               "/ProcessResultFile/ProductMetaData/ObservationStartDateTime"
#define     TAG_PRDCT_OBS_END_DATE_TIME                 "/ProcessResultFile/ProductMetaData/ObservationEndDateTime"
#define     TAG_PRDCT_GRING_POINT_LATITUDE              "/ProcessResultFile/ProductMetaData/GringPointLatitude"
#define     TAG_PRDCT_GRING_POINT_LONGITUDE             "/ProcessResultFile/ProductMetaData/GringPointLongitude"
#define     TAG_PRDCT_PGE_NAME                          "/ProcessResultFile/ProductMetaData/PGEName"
#define     TAG_PRDCT_INPUT_FILE_NAME                   "/ProcessResultFile/ProductMetaData/InputFileName"
#define     TAG_PRDCT_PROCESSING_CENTER                 "/ProcessResultFile/ProductMetaData/ProcessingCenter"
#define     TAG_PRDCT_CONTACT_ORGANIZATION_NAME         "/ProcessResultFile/ProductMetaData/ContactOrganizationName"
#define     TAG_PRDCT_CONTACT_ORGANIZATION_TEL          "/ProcessResultFile/ProductMetaData/ContactOrganizationTelephone"
#define     TAG_PRDCT_START_ORBIT_NUMBER                "/ProcessResultFile/ProductMetaData/StartOrbitNumber"
#define     TAG_PRDCT_STOP_ORBIT_NUMBER                 "/ProcessResultFile/ProductMetaData/StopOrbitNumber"
#define     TAG_PRDCT_EQUATOR_CROSS_LONGITUDE           "/ProcessResultFile/ProductMetaData/EquatorCrossingLongitude"
#define     TAG_PRDCT_EQUATOR_CROSS_DATE_TIME           "/ProcessResultFile/ProductMetaData/EquatorCrossingDateTime"
#define     TAG_PRDCT_ORBIT_DIRECTION                   "/ProcessResultFile/ProductMetaData/OrbitDirection"
#define     TAG_PRDCT_PASS_NUMBER                       "/ProcessResultFile/ProductMetaData/PassNumber"
#define     TAG_PRDCT_ORBIT_DATA_FILE_NAME              "/ProcessResultFile/ProductMetaData/OrbitDataFileName"
#define     TAG_PRDCT_EPHEMERIS_MISSING_DATA_RATE       "/ProcessResultFile/ProductMetaData/EphemerisMissingDataRate"
#define     TAG_PRDCT_ATTITUDE_MISSING_DATA_RATE        "/ProcessResultFile/ProductMetaData/AttitudeMissingDataRate"
#define     TAG_PRDCT_ORBIT_DATA_TYPE                   "/ProcessResultFile/ProductMetaData/OrbitDataType"
#define     TAG_PRDCT_PLATFORM_SHORT_NAME               "/ProcessResultFile/ProductMetaData/PlatformShortName"
#define     TAG_PRDCT_SENSOR_SHORT_NAME                 "/ProcessResultFile/ProductMetaData/SensorShortName"
#define     TAG_PRDCT_NUMBER_OF_SCANS                   "/ProcessResultFile/ProductMetaData/NumberOfScans"
#define     TAG_PRDCT_NUMBER_OF_MISSING_SCANS           "/ProcessResultFile/ProductMetaData/NumberOfMissingScans"
#define     TAG_PRDCT_TACHO_PULSE_COUNT                 "/ProcessResultFile/ProductMetaData/AntennaRotationVelocity"
#define     TAG_PRDCT_ECS_DATA_MODEL                    "/ProcessResultFile/ProductMetaData/ECSDataModel"
#define     TAG_PRDCT_NUMBER_OF_PACKETS                 "/ProcessResultFile/ProductMetaData/NumberOfPackets"
#define     TAG_PRDCT_NUMBER_OF_INPUT_FILES             "/ProcessResultFile/ProductMetaData/NumberOfInputFiles"
#define     TAG_PRDCT_NUMBER_OF_MISSING_PACKETS         "/ProcessResultFile/ProductMetaData/NumberMissingPackets"
#define     TAG_PRDCT_NUMBER_OF_GOOD_PACKETS            "/ProcessResultFile/ProductMetaData/NumberOfGoodPackets"
#define     TAG_PRDCT_OVER_LAP_SCANS                    "/ProcessResultFile/ProductMetaData/OverlapScans"
#define     TAG_PRDCT_QA_LOCATION_OF_PACKET_DISCONTINUITRY  "/ProcessResultFile/ProductMetaData/QALocationOfPacketDiscontinuity"
#define     TAG_PRDCT_EPHEMERIS_QA                      "/ProcessResultFile/ProductMetaData/EphemerisQA"
#define     TAG_PRDCT_AUTOMATIC_QA_FLAG                 "/ProcessResultFile/ProductMetaData/AutomaticQAFlag"
#define     TAG_PRDCT_SCIENCE_QUALITY_FLAG              "/ProcessResultFile/ProductMetaData/ScienceQualityFlag"
#define     TAG_PRDCT_SCIENCE_QUALITY_FLAG_EXPLANATION  "/ProcessResultFile/ProductMetaData/ScienceQualityFlagExplanation"
#define     TAG_PRDCT_AUTOMATIC_QA_FLAG_EXPLANATION     "/ProcessResultFile/ProductMetaData/AutomaticQAFlagExplanation"
#define     TAG_PRDCT_QA_PERCENT_MISSING_DATA           "/ProcessResultFile/ProductMetaData/QAPercentMissingData"
#define     TAG_PRDCT_QA_PERCENT_OUT_OF_BOUNDS_DATA     "/ProcessResultFile/ProductMetaData/QAPercentOutOfBoundsData"
#define     TAG_PRDCT_QA_PERCENT_PARITY_ERROR_DATA      "/ProcessResultFile/ProductMetaData/QAPercentParityErrorData"
#define     TAG_PRDCT_PROCESSING_QA_DESCRIPTION         "/ProcessResultFile/ProductMetaData/ProcessingQADescription"
#define     TAG_PRDCT_PROCESSING_QA_ATTIRBUTE           "/ProcessResultFile/ProductMetaData/ProcessingQAAttribute"
#define     TAG_PRDCT_GLOBAL_METEOROLOGICAL_DATA_TYPE   "/ProcessResultFile/ProductMetaData/GlobalMeteorologicalDataType"
#define     TAG_PRDCT_ANCILLARY_DATA_INFOMATION         "/ProcessResultFile/ProductMetaData/AncillaryDataInformation"
#define     TAG_PRDCT_SATELLITE_ORBIT                   "/ProcessResultFile/ProductMetaData/SatelliteOrbit"
#define     TAG_PRDCT_SATELLITE_ALTITUDE                "/ProcessResultFile/ProductMetaData/SatelliteAltitude"
#define     TAG_PRDCT_ORBIT_SEMI_MAJOR_AXIS             "/ProcessResultFile/ProductMetaData/OrbitSemiMajorAxis"
#define     TAG_PRDCT_ORBIT_ECCENTRICITY                "/ProcessResultFile/ProductMetaData/OrbitEccentricity"
#define     TAG_PRDCT_ORBIT_ARGUMENT_PERIGEE            "/ProcessResultFile/ProductMetaData/OrbitArgumentPerigee"
#define     TAG_PRDCT_ORBIT_INCLIATION                  "/ProcessResultFile/ProductMetaData/OrbitInclination"
#define     TAG_PRDCT_ORBIT_PERIOD                      "/ProcessResultFile/ProductMetaData/OrbitPeriod"
#define     TAG_PRDCT_REVISIT_TIME                      "/ProcessResultFile/ProductMetaData/RevisitTime"
#define     TAG_PRDCT_ALGORITHM_DEVEROPER               "/ProcessResultFile/ProductMetaData/AlgorithmDeveloper"
#define     TAG_PRDCT_AMSR_CHANNEL                      "/ProcessResultFile/ProductMetaData/AMSRChannel"
#define     TAG_PRDCT_AMSR_BAND_WIDTH                   "/ProcessResultFile/ProductMetaData/AMSRBandWidth"
#define     TAG_PRDCT_AMSR_E_BEAM_WIDTH                 "/ProcessResultFile/ProductMetaData/AMSRBeamWidth"
#define     TAG_PRDCT_OFF_NADIR                         "/ProcessResultFile/ProductMetaData/OffNadir"
#define     TAG_PRDCT_SPATIAL_RESOLUTION                "/ProcessResultFile/ProductMetaData/SpatialResolution"
#define     TAG_PRDCT_SCANNING_RERIOD                   "/ProcessResultFile/ProductMetaData/ScanningPeriod"
#define     TAG_PRDCT_SWATH_WIDTH                       "/ProcessResultFile/ProductMetaData/SwathWidth"
#define     TAG_PRDCT_DYNAMIC_RANGE                     "/ProcessResultFile/ProductMetaData/DynamicRange"
#define     TAG_PRDCT_DATA_FORMAT_TYPE                  "/ProcessResultFile/ProductMetaData/DataFormatType"
#define     TAG_PRDCT_HDF_FORMAT_VERSION                "/ProcessResultFile/ProductMetaData/HDFFormatVersion"
#define     TAG_PRDCT_ELLIPSID_NAME                     "/ProcessResultFile/ProductMetaData/EllipsoidName"
#define     TAG_PRDCT_SEMI_MAJOR_AXIS_OF_EARTH          "/ProcessResultFile/ProductMetaData/SemiMajorAxisofEarth"
#define     TAG_PRDCT_FLATTENING_RATIO_OF_EARTH         "/ProcessResultFile/ProductMetaData/FlatteningRatioofEarth"
#define     TAG_PRDCT_SENSOR_ALIGNMENT                  "/ProcessResultFile/ProductMetaData/SensorAlignment"

#define     TAG_PRDCT_THERMISTOR1_COUNT_RANGE           "/ProcessResultFile/ProductMetaData/Thermistor1CountRange"
#define     TAG_PRDCT_THERMISTOR1_CONVERSION_TABLE_D    "/ProcessResultFile/ProductMetaData/Thermistor1ConversionTableD"
#define     TAG_PRDCT_THERMISTOR1_CONVERSION_TABLE_E    "/ProcessResultFile/ProductMetaData/Thermistor1ConversionTableE"
#define     TAG_PRDCT_THERMISTOR1_CONVERSION_TABLE_F    "/ProcessResultFile/ProductMetaData/Thermistor1ConversionTableF"
#define     TAG_PRDCT_THERMISTOR2_COUNT_RANGE           "/ProcessResultFile/ProductMetaData/Thermistor2CountRange"
#define     TAG_PRDCT_THERMISTOR2_CONVERSION_TABLE_W4   "/ProcessResultFile/ProductMetaData/Thermistor2ConversionTableW4"
#define     TAG_PRDCT_THERMISTOR2_CONVERSION_TABLE_W3   "/ProcessResultFile/ProductMetaData/Thermistor2ConversionTableW3"
#define     TAG_PRDCT_THERMISTOR2_CONVERSION_TABLE_W2   "/ProcessResultFile/ProductMetaData/Thermistor2ConversionTableW2"
#define     TAG_PRDCT_THERMISTOR2_CONVERSION_TABLE_W1   "/ProcessResultFile/ProductMetaData/Thermistor2ConversionTableW1"
#define     TAG_PRDCT_THERMISTOR2_CONVERSION_TABLE_W0   "/ProcessResultFile/ProductMetaData/Thermistor2ConversionTableW0"
#define     TAG_PRDCT_THERMISTOR3_COUNT_RANGE           "/ProcessResultFile/ProductMetaData/Thermistor3CountRange"
#define     TAG_PRDCT_THERMISTOR3_CONVERSION_TABLE_W4   "/ProcessResultFile/ProductMetaData/Thermistor3ConversionTableW4"
#define     TAG_PRDCT_THERMISTOR3_CONVERSION_TABLE_W3   "/ProcessResultFile/ProductMetaData/Thermistor3ConversionTableW3"
#define     TAG_PRDCT_THERMISTOR3_CONVERSION_TABLE_W2   "/ProcessResultFile/ProductMetaData/Thermistor3ConversionTableW2"
#define     TAG_PRDCT_THERMISTOR3_CONVERSION_TABLE_W1   "/ProcessResultFile/ProductMetaData/Thermistor3ConversionTableW1"
#define     TAG_PRDCT_THERMISTOR3_CONVERSION_TABLE_W0   "/ProcessResultFile/ProductMetaData/Thermistor3ConversionTableW0"
#define     TAG_PRDCT_PLATINUM1_COUNT_RANGE             "/ProcessResultFile/ProductMetaData/Platinum1CountRange"
#define     TAG_PRDCT_PLATINUM1_CONVERSION_TABLE_W4     "/ProcessResultFile/ProductMetaData/Platinum1ConversionTableW4"
#define     TAG_PRDCT_PLATINUM1_CONVERSION_TABLE_W3     "/ProcessResultFile/ProductMetaData/Platinum1ConversionTableW3"
#define     TAG_PRDCT_PLATINUM1_CONVERSION_TABLE_W2     "/ProcessResultFile/ProductMetaData/Platinum1ConversionTableW2"
#define     TAG_PRDCT_PLATINUM1_CONVERSION_TABLE_W1     "/ProcessResultFile/ProductMetaData/Platinum1ConversionTableW1"
#define     TAG_PRDCT_PLATINUM1_CONVERSION_TABLE_W0     "/ProcessResultFile/ProductMetaData/Platinum1ConversionTableW0"
#define     TAG_PRDCT_PLATINUM2_COUNT_RANGE             "/ProcessResultFile/ProductMetaData/Platinum2CountRange"
#define     TAG_PRDCT_PLATINUM2_CONVERSION_TABLE_W4     "/ProcessResultFile/ProductMetaData/Platinum2ConversionTableW4"
#define     TAG_PRDCT_PLATINUM2_CONVERSION_TABLE_W3     "/ProcessResultFile/ProductMetaData/Platinum2ConversionTableW3"
#define     TAG_PRDCT_PLATINUM2_CONVERSION_TABLE_W2     "/ProcessResultFile/ProductMetaData/Platinum2ConversionTableW2"
#define     TAG_PRDCT_PLATINUM2_CONVERSION_TABLE_W1     "/ProcessResultFile/ProductMetaData/Platinum2ConversionTableW1"
#define     TAG_PRDCT_PLATINUM2_CONVERSION_TABLE_W0     "/ProcessResultFile/ProductMetaData/Platinum2ConversionTableW0"
#define     TAG_PRDCT_PLATINUM3_CONVERSION_TABLE_W4     "/ProcessResultFile/ProductMetaData/Platinum3ConversionTableW4"
#define     TAG_PRDCT_PLATINUM3_CONVERSION_TABLE_W3     "/ProcessResultFile/ProductMetaData/Platinum3ConversionTableW3"
#define     TAG_PRDCT_PLATINUM3_CONVERSION_TABLE_W2     "/ProcessResultFile/ProductMetaData/Platinum3ConversionTableW2"
#define     TAG_PRDCT_PLATINUM3_CONVERSION_TABLE_W1     "/ProcessResultFile/ProductMetaData/Platinum3ConversionTableW1"
#define     TAG_PRDCT_PLATINUM3_CONVERSION_TABLE_W0     "/ProcessResultFile/ProductMetaData/Platinum3ConversionTableW0"

#define     TAG_PRDCT_COEFFICIENT_AVV                   "/ProcessResultFile/ProductMetaData/CoefficientAvv"
#define     TAG_PRDCT_COEFFICIENT_AHV                   "/ProcessResultFile/ProductMetaData/CoefficientAhv"
#define     TAG_PRDCT_COEFFICIENT_AOV                   "/ProcessResultFile/ProductMetaData/CoefficientAov"
#define     TAG_PRDCT_COEFFICIENT_AHH                   "/ProcessResultFile/ProductMetaData/CoefficientAhh"
#define     TAG_PRDCT_COEFFICIENT_AVH                   "/ProcessResultFile/ProductMetaData/CoefficientAvh"
#define     TAG_PRDCT_COEFFICIENT_AOH                   "/ProcessResultFile/ProductMetaData/CoefficientAoh"
#define     TAG_PRDCT_CSM_TEMPERATURE                   "/ProcessResultFile/ProductMetaData/CSMTemperature"
#define     TAG_PRDCT_COREGISTRATION_PARAMETERER_A1     "/ProcessResultFile/ProductMetaData/CoRegistrationParameterA1"
#define     TAG_PRDCT_COREGISTRATION_PARAMETERER_A2     "/ProcessResultFile/ProductMetaData/CoRegistrationParameterA2"
#define     TAG_PRDCT_CALIBRATION_CURVE_COEFFICIENT1    "/ProcessResultFile/ProductMetaData/CalibrationCurveCoefficient1"
#define     TAG_PRDCT_CALIBRATION_CURVE_COEFFICIENT2    "/ProcessResultFile/ProductMetaData/CalibrationCurveCoefficient2"
#define     TAG_PRDCT_CALIBRATION_CURVE_COEFFICIENT3    "/ProcessResultFile/ProductMetaData/CalibrationCurveCoefficient3"
#define     TAG_PRDCT_CALIBRATION_CURVE_COEFFICIENT4    "/ProcessResultFile/ProductMetaData/CalibrationCurveCoefficient4"
#define     TAG_PRDCT_CALIBRATION_CURVE_COEFFICIENT5    "/ProcessResultFile/ProductMetaData/CalibrationCurveCoefficient5"
#define     TAG_PRDCT_CALIBRATION_METHOD                "/ProcessResultFile/ProductMetaData/CalibrationMethod"
#define     TAG_PRDCT_MEAN_TYPE                         "/ProcessResultFile/ProductMetaData/MeanType"
#define     TAG_PRDCT_PROJECTION                        "/ProcessResultFile/ProductMetaData/Projection"
#define     TAG_PRDCT_RESOLUTION                        "/ProcessResultFile/ProductMetaData/Resolution"
#define     TAG_PRDCT_ALGORISM_DEVELOPER                "/ProcessResultFile/ProductMetaData/AlgorithmDeveloper"
#define     TAG_POSITION                                "/ProcessResultFile/Position/Position"
#define     TAG_LATITUDE                                "/Latitude"
#define     TAG_LONGITUDE                               "/Longitude"
#define     TAG_BROWS                                   "/ProcessResultFile/ProcessResultInformation/ProductResult/Browse"
#define     TAG_THUMNAIL                                "/ProcessResultFile/ProcessResultInformation/ProductResult/Thumbnail"
#define     TAG_PRODUCT_FILE_NAME                       "/ProcessResultFile/ProcessResultInformation/ProductResult/ProductFileName"
#define     TAG_ERR_CODE                                "/ProcessResultFile/ProcessResultInformation/ProductResult/DistributeErrorCode"
#define     TAG_COMMENT                                 "/ProcessResultFile/ProcessResultInformation/ProductResult/OperatorComment"
#define     TAG_ANTENNA_ROTATION_QA                     "/ProcessResultFile/ProcessResultInformation/ErrorCheck/AntennaRotationQA"
#define     TAG_HTS_QA                                  "/ProcessResultFile/ProcessResultInformation/ErrorCheck/HotCalibrationSourceQA"
#define     TAG_ATTITUDE_DATA_QA                        "/ProcessResultFile/ProcessResultInformation/ErrorCheck/AttitudeDataQA"
#define     TAG_EPHEMERIS_DATA_QA                       "/ProcessResultFile/ProcessResultInformation/ErrorCheck/EphemerisDataQA"
#define     TAG_GEO_INFO_QA                             "/ProcessResultFile/ProcessResultInformation/ErrorCheck/QualityofGeometricInformationQA"
#define     TAG_BT_QA                                   "/ProcessResultFile/ProcessResultInformation/ErrorCheck/BrightnessTemperatureQA"
#define     TAG_MISSING_DATA_QA                         "/ProcessResultFile/ProcessResultInformation/ErrorCheck/MissingDataQA"
#define     TAG_SYS_VERSION                             "/ProcessResultFile/ProcessResultInformation/Version/SystemVersion"
#define     TAG_PRM_VERSION                             "/ProcessResultFile/ProcessResultInformation/Version/ParameterVersion"
#define     TAG_ALG_VERSION                             "/ProcessResultFile/ProcessResultInformation/Version/AlgorithmVersion"

#define     TAG_WORKORDER_ID                            "/ProcessResultFile/ProcessResultInformation/AdditionalInformation/WorkorderID"
#define     TAG_DATAPROCESS_NAME                        "/ProcessResultFile/ProcessResultInformation/AdditionalInformation/DataprocessName"
#define     TAG_PLANNING_DATE_TIME                      "/ProcessResultFile/ProcessResultInformation/AdditionalInformation/PlanningDateTime"
#define     TAG_PROCESS_START_TIME                      "/ProcessResultFile/ProcessResultInformation/AdditionalInformation/ProcessStartTime"
#define     TAG_PROCESS_STOP_TIME                       "/ProcessResultFile/ProcessResultInformation/AdditionalInformation/ProcessStopTime"

#define     PROCESS_START_TIME_FILENAME                 "process_start_time.txt"
/*------------------------------------------------------------------*/

#define SHIFT_BIT_24                        24
#define SHIFT_BIT_16                        16
#define SHIFT_BIT_8                         8

#define MASK_BIT_16                         0xFFFF
#define MASK_BIT_12                         0x0FFF
#define MASK_BIT_8                          0xFF
#define MASK_PARITY                         0xC0

#define ON                                  1
#define OFF                                 0

#define PACKET_NO_1                         0
#define PACKET_NO_2                         1
#define PACKET_NO_3                         2
#define PACKET_NO_4                         3
#define PACKET_NO_5                         4
#define PACKET_NO_6                         5
#define PACKET_NO_7                         6
#define PACKET_NO_8                         7
#define PACKET_NO_9                         8
#define PACKET_NO_10                        9
#define PACKET_NO_11                        10
#define PACKET_NO_12                        11
#define PACKET_NO_13                        12
#define PACKET_NO_14                        13
#define PACKET_NO_15                        14
#define PACKET_NO_16                        15

#define PACKET_NUM                          16

#define CHK_OK                              0
#define CHK_NG                              1

#define FRQ_L_NUM                           6
#define FRQ_H_NUM                           2

#define LOOP_NUM_1                          1
#define LOOP_NUM_2                          2

#define PCD_S_POS                           16
#define PCD_DATA_SIZE                       64

#define SPC_COUNT_NUM                       34
#define SPC_COUNT_S_POS                     112
#define SPS_COUNT_NUM                       46
#define SPS_COUNT_S_POS                     600

#define SUPPLEMENT_SIZE                     248

#define PIXEL_NUM_IN_PACKET                 25
#define PIXEL_NUM_IN_PACKET_NO_11           18

#define LOW_PIXEL_NUM_CSM_HTS               16

#define NEXT_PIXEL_POS                      40
#define NEXT_PIXEL_POS_89                   10

#define LOW_PIXEL_NUM                       243
#define HIGH_PIXEL_NUM                      486
#define LOW_PIXEL_NUM_CSM_HTS               16
#define HIGH_PIXEL_NUM_CSM_HTS              32

#define STS_NO_GAP                          0
#define STS_GAP_REV                         1
#define STS_GAP_OVER                        2

#define DETECT_ON                           1
#define DETECT_OFF                          0

#define PACKET_SIZE                         1024
#define FNAME_SIZE                          32

//#define ATTITUDE_DATA_NUM                   6
//#define NAVIGATION_DATA_NUM                 6

#define DOT_NUM                             4
#define NO_EXIST                            -1
#define NAVI_FLAG_KALMAN_FILTER             2
#define FLG_CONTAIN_MISS_DATA               1

#define NAVIGATION_STOP                     0
#define ALTITUDE_CHECK_ARG_NUM              26
#define SATELLITY_VEL_CHECK_ARG_NUM         27

#define DSEF_EXTRAPOLATE                    1

#define QUALITY_OK                          0
#define QUALITY_NG                          1

#define COPY_BUFF_SIZE                      1024
#define PROCESS_DONE_FILE_NAME              "l1pre_proc_done.csv"
#define FILE_BACK_NAME                      ".bak"
#define FILE_LOCK_NAME                      ".lock"
#define SN_LIMIT_SIZE_SEC                   11856
#define TIME_OUT_MAX_CNT                    6
#define TIME_OUT_SEC                        10
#define NEARLY_ZERO                         0.000000001

#define MAX_RECORD_NUM                      40360
#define S_H_POS                             7
#define FLAG_89GHZ_ON                       1
#define FLAG_89GHZ_OFF                      0

#define PCD_NAVI_STS_POS                    56
#define BIT_NUM_16                          16
#define MASK_NAVI_STS                       0x0003
#define MAX_DATA_SIZE                       256

#define RECORD_NO_POS                       53
#define RECORD_NO_LEN                       5
#define START_DAY_POS                       59
#define START_DAY_LEN                       8
#define END_DAY_POS                         68
#define END_DAY_LEN                         8
#define START_TIME_DATA_POS                 21
#define START_TIME_DATA_LEN                 21
#define END_TIME_DATA_POS                   43
#define END_TIME_DATA_LEN                   21
#define PSC_DATA_POS                        65
#define PSC_DATA_LEN                        13
#define T_REF_DATA_POS                      79
#define T_REF_DATA_LEN                      11
#define T_GREF_UTC_POS                      91
#define T_GREF_UTC_LEN                      21
#define LAST_RECORD_STR                     "99999999 99:99:99.999"

#define GPS_TAI_SEC                         19

#define STS_DMS                             0
#define STS_DMS_GPS                         1
#define STS_GPS                             2
#define INTEGRATION_VAL                     0.0013

#define FILENAME_LENGTH                     512
#define SEQ_TIMER_POS                       152

#define DL_ORBIT_NUM_MAX                        10
#define ORBIT_PERIOD                            98.8

/*#define COMPLEX_IMAGE_MAX_LIMIT               65530
#define COMPLEX_IMAGE_MIN_LIMIT             -32760 */

#define BROWSE                                  "Browse"
#define THUMBNAIL                               "Thumbnail"
#define TAG_BROWSE_THUMBNAIL_FILE               "config_browse_thumbnail.xml"

/*------------------------------------------------------------------*/
#define CHANGE_ENGI_VALUE_LOG_RANGE 200

#define MOON_PLANET 9
#define SUN_PLANET  2

#ifdef PI
#undef PI
#endif
#define PI          3.14159265358979323846
#define TWOPI       6.28318530717958647693

#define DEG_PER_RAD 5.72957795130823208768e1
#define RAD_PER_DEG 1.74532925199432957692e-2

#ifdef RADDEG
#undef RADDEG
#endif
#define RADDEG(x)   ( (x) * DEG_PER_RAD )

#ifdef DEGRAD
#undef DEGRAD
#endif
#define DEGRAD(x)   ( (x) * RAD_PER_DEG )
#define FRQ_L_NUM   6
#define FRQ_H_NUM   2
#define FREQUENCY   8

/* am2_com_para.h  */
/* am2_com_para_20100419.h  */
typedef struct {
    double  geo_obsvector[2][FREQUENCY];
    double  geo_reflector_offset_angle;
    double  geo_rot_alignment[3][3];
    double  geo_antenna_rot_angle_offset;
    double  geo_sensor_alignment_phi;
    double  geo_sensor_alignment_theta;
    double  geo_sensor_alignment_psi;
    double  geo_sensor_origin_x;
    double  geo_sensor_origin_y;
    double  geo_sensor_origin_z;
    double  geo_registration[FREQUENCY][2];
} com_para_geo;


/*------------------------------------------------------------------*/

#define ON 1
#define OFF 0
#define MIN(a,b) ( ((a)<(b) ) ? (a) : (b) )
#define SAME_VALUE(a,b) ( (fabs((a)-(b)) < NEARLY_ZERO) ? true : false )
#define SIGN(a) ( ((a)<0) ? -1 : 1 )
#define ERROR_VALUE -99999.9

#define POLARIZATION 2
#define LOW_PIXEL_NUM 243
#define HIGH_PIXEL_NUM 486
#define LOW_PIXEL_NUM_CSM_HTS 16
#define HIGH_PIXEL_NUM_CSM_HTS 32
#define LOW_FREQUENCY_NUM 6
#define HIGH_FREQUENCY_NUM 2

#define CSM_STAYLIGHT_CONDITION_NUM 30
#define CSM_RFI_CONDITION_NUM 30
#define HTS_RFI_CONDITION_NUM 30

#define HTS_TEMP_NUM 10
#define HTS_TEMP_START 6
#define HTS_TEMP_END 15

#define PIXEL_NUM PIXEL_NUM_H
#define PIXEL_NUM_CSM_HTS HIGH_PIXEL_NUM_CSM_HTS

#define LEN_FILE 512
#define LEN_PATH 1024
#define LEN_ID 128

#define SAMPLE_FREQUENCY 0
#define SAMPLE_POLARIZATION 0
#define SAMPLE_PIXEL 15

#define SCHEMA_FILENAME_CPARA_EX                        "./schema_ex.xml"
#define SCHEMA_FILENAME_CPARA_IN                        "./schema_in.xml"
#define SCHEMA_FILENAME_CPARA_EX_PERIOD                 "./schema_ex_period.xml"

#define COM_PARAM_FILENAME_OBSTIME_SPS_TIMER            "obstime_sps_timer.csv"
#define COM_PARAM_FILENAME_OBSTIME_START_OFFSET         "obstime_start_offset.csv"
#define COM_PARAM_FILENAME_OBSTIME_SQ_TIMER_OFFSET      "obstime_sq_timer_offset.csv"
#define COM_PARAM_FILENAME_OBSTIME_DELAY_TIME           "obstime_delay_time.csv"

#define COM_PARAM_FILENAME_GEO_OBS_VECTOR               "geo_obs_vector.csv"
#define COM_PARAM_FILENAME_GEO_ROT_ALIGNMENT_MATRIX     "geo_rot_alignment_matrix.csv"
#define COM_PARAM_FILENAME_GEO_REGISTRATION             "geo_registration.csv"

#define COM_PARAM_FILENAME_RAD                          "rad.csv"

#define SCHEMA_FILENAME_L1_WORK_ORDER                   "./schema_l1.xml"





#define TAG_CONFIG_SCHEMADIRECTORYINFO_L0_WO            "/DataProcessingDeviceParaFileInfo/SchemaDirectoryInfo/L0WorkOrder"

#define TAG_CONFIG_SCHEMADIRECTORYINFO_L1_WO            "/DataProcessingDeviceParaFileInfo/SchemaDirectoryInfo/L1WorkOrder"

#define TAG_CONFIG_SCHEMADIRECTORYINFO_L2_WO            "/DataProcessingDeviceParaFileInfo/SchemaDirectoryInfo/L2WorkOrder"

#define TAG_CONFIG_SCHEMADIRECTORYINFO_L3_WO            "/DataProcessingDeviceParaFileInfo/SchemaDirectoryInfo/L3WorkOrder"


#define L1PRODUCT_SCAN_NUM_MIN (4)

/*------------------------------------------------------------------*
 * Definition of Structure.                                         *
 *------------------------------------------------------------------*/

/** Leapsecond 1993/01/01 00:00:00. */
#define LEAPSECOND_TAI93_BEGIN (27)
#define JD_TAI93_BEGIN         (2448988.5)
#define SECOND_PER_1DAY        (24*60*60)

/** subscript of leapsecond table . */
enum g_leap_second_table_subscript
{
    TAI93,
    MJD,
    LEAPSECOND,
    LEAPSECOND_COL_NUM
};

/** GPS structure. */
typedef struct
{
    int gps_week;      /* GPS WEEK */
    int gps_sec;       /* GPS SECOND */
    int gps_fine_time; /* GPS MICRO SECOND */
} gps;

/** UTC structure. */
typedef struct
{
    int year;          /* UTC YEAR */
    int month;         /* 01...12 */
    int day;           /* 01...31 */
    int hour;          /* 00...23 */
    int minute;        /* 00...59 */
    int second;        /* 00...60 */
    int microsecond;   /* 000...999999 */
} utc;

/*------------------------------------------------------------------*
 * Definition of Function.                                          *
 *------------------------------------------------------------------*/
 
/** Get Leapsecond.
 * Leapsecond is acquired and it maintains it in the memory.
 * 
 * @param  leap_second_file_name [in]FILE NAME OF LEAPSECOND FILE.
 * @return NORMAL
 * @return FOPEN_ERROR_LEAPSECOND
 * @return MEM_ERROR_LEAPSECOND
 * @return NO_EXIST_ERROR_LEAPSECOND_TABLE
 */
#if 0
int com_get_leapsecond(const char *leap_second_file_name);
#endif
int com_get_leapsecond_formal(const char *leap_second_file_name);
int com_get_leapsecond_default(void);

/** Get LEAPSECOND(TAI-UTC).
 * (Internal function)
 * @param subscript             [in]TAI93 or MJD
 * @param time                  [in]TAI93 TIME or MJD TIME
 * @param p_leapsecond_utc_tai  [out]LEAPSECOND(TAI-UTC)
 * @param p_is_just_leapsecond  [out]true:JUST LEAPSECOND(subscript:TAI93)
 * @return true : exists
 * @return false: no exists
 */
bool get_leapsecond_utc_tai(
    short subscript,
    double time,
    double *p_leapsecond_utc_tai,
    bool *p_is_just_leapsecond);

/** Release Leapsecond Table.
 */
void com_release_leapsecond(void);


/** TAI93 TIME -> UTC TIME.
 * It converts it from TAI93 TIME to UTC TIME.
 * 
 * @param tai93_time  [in]TAI93 TIME 
 * @param p_utc_time  [out]UTC TIME 
 * @return NORMAL
 * @return NO_EXIST_ERROR_LEAPSECOND_TABLE
 * @return TIME_ERROR_INPUT_TIME
 * @return TIME_ERROR_LEAPSECOND_TABLE
 */
int com_time_tai93_utc(double tai93_time, utc *p_utc_time);

/** UTC TIME -> TAI93 TIME.
 * It converts it from UTC TIME to TAI93 TIME.
 * 
 * @param p_utc_time    [in]UTC TIME 
 * @param p_tai93_time  [out]TAI93 TIME 
 * @return NORMAL
 * @return TIME_ERROR_INPUT_TIME
 * @return TIME_ERROR_LEAPSECOND_TABLE
 */
int com_time_utc_tai93(const utc *p_utc_time, double *p_tai93_time);

/** Get Julian Day and Modified Julian Day.
 * 
 * @param year  [in] UTC Year
 * @param month [in] UTC month
 * @param day   [in] UTC day
 * @param p_jd  [out]Julian Day
 * @param p_mjd [out]Modified Julian Day
 */
void get_jd_and_mjd(
    int year,
    int month,
    int day,
    double *p_jd,
    double *p_mjd);

#endif //PUBLIC_LEAPSEC end
