!---------------------------------------------------------------------
!  Copyright (c) NEC Corporation 2015 All rights reserved.           l
!  File Name    : sample1.f95                                        l
!  Anstract     : SGTK toolkit sample program                        l
!  Author       : NEC                                                l
!  Date         : 2015/03/20                                         l
!  Version      : 1.00                                               l
!---------------------------------------------------------------------
!
!  Ver  0.1 (specified in 2015.03.09
!            and updateded for SGTK_set_dims parameters)
!  Ver  0.2 (specified in 2015.03.15)
!
      PROGRAM sample

      IMPLICIT NONE

      INCLUDE "SGTK_f.h"

!  Identification 
      CHARACTER(LEN=40):: fname = 'product_sample'
      INTEGER,PARAMETER:: rank = 6
      INTEGER:: hdf5_id
      CHARACTER(LEN=40):: configfile = 'product_sample_config'

!  Define time variables
      INTEGER:: year_now, month_now, day_now, hour_now, minutes_now, second_now, mills_now
      INTEGER:: datetime_values(1:8)
      REAL*8:: tai93_value
      CHARACTER(LEN=8):: date0_now
      CHARACTER(LEN=10):: time0_now
      CHARACTER(LEN=5):: zone0_now

!  Define Location Variables
      INTEGER,PARAMETER:: latlons = 5
      CHARACTER(LEN=40):: granule_id
      DATA granule_id/'Granule-ID-AAAA1111111'/
      REAL*4:: latlon_get(1:2*latlons)

!  Define Dataset format
      INTEGER:: dimsizes(1:rank)
      DATA dimsizes/ 4, 3, 2, 3, 2, 2 /
      CHARACTER(LEN=10):: dimlabels(1:rank)
      DATA dimlabels/'row', 'column', 'layer', 'pairs', 'passage', 'street'/
      INTEGER,PARAMETER:: dataset_type = 1
      CHARACTER(LEN=20):: access_label
      DATA access_label/'sample dataset'/
      INTEGER,PARAMETER:: value_kind = 2

!  Set Dataset values
      INTEGER:: ds1_pattern(1:4,1:3,1:2), ds1(1:4,1:3,1:2,1:3,1:2,1:2)
      INTEGER*1:: ds2_pattern(1:4,1:3,1:2), ds2(1:4,1:3,1:2,1:3,1:2,1:2)
      INTEGER*2:: ds3_pattern(1:4,1:3,1:2), ds3(1:4,1:3,1:2,1:3,1:2,1:2)
      INTEGER*4:: ds4_pattern(1:4,1:3,1:2), ds4(1:4,1:3,1:2,1:3,1:2,1:2)
      INTEGER:: ds_factor(1:3,1:2,1:2)
      INTEGER:: factor
      DATA ds1_pattern/ 1,2,3,4, 5,6,7,8, 9,10,11,12, &
                        -1,-2,-3,-4, -5,-6,-7,-8, -9,-10,-11,-12 /
      DATA ds2_pattern/ 1,2,3,4, 5,6,7,8, -1,-2,-3,-4, &
                        124,125,126,127, -125,-126,-127,-128, 10,20,30,40 /
      DATA ds3_pattern/ 0,1,2,3, 128,256,512,1024, 2048,4096,8192,16384, &
                        32764,32765,32766,32767, -1,-2,-3,-4, -128,-256,-512,-4096 /
      DATA ds4_pattern/ 0,1,2,3, 4096,8192,16384,32768, 65536,131072,262144,524288, &
                        -1,-2,-3,-4, -4096,-8192,-16384,-32768, 0,0,0,2147483647 /
      DATA ds_factor/ 1,2,4, 2,4,8, 4,8,16, 8,16,32 /
      REAL:: fds1_pattern(1:4,1:3,1:2), fds1(1:4,1:3,1:2,1:3,1:2,1:2)
      REAL*4:: fds2_pattern(1:4,1:3,1:2), fds2(1:4,1:3,1:2,1:3,1:2,1:2)
      REAL*8:: fds3_pattern(1:4,1:3,1:2), fds3(1:4,1:3,1:2,1:3,1:2,1:2)
      REAL:: fds_factor(1:3,1:2,1:2)
      REAL:: f_factor
      DATA fds1_pattern/ 1.,2.,3.,4., 5.,6.,7.,8., -1.,-2.,-3.,-4., &
                         -5.,-6.,-7.,-8., 1.,1.,1.,1., 1.,2.,2.,2. /
      DATA fds2_pattern/ 2.,2.,2.,2., 1.,2.,2.,2., 1.,1.,2.,2., &
                         1.,1.,1.,2., 1.,1.,1.,3., 1.,1.,1.,4. /
      DATA fds3_pattern/ 2.,-2.,2.,2., 1.,2.,-2.,2., 1.,1.,2.,-2., &
                         1.,-1.,1.,2., 1.,1.,-1.,3., 1.,1.,1.,-4. /
      DATA fds_factor/ 1.0,2.0,4.0, 2.0,4.0,8.0, 3.0,6.0,12.0, 4.0,8.0,16.0 /

!  Set meta data value
      CHARACTER(LEN=30):: meta_name
      DATA meta_name/'sample meta name'/
      CHARACTER(LEN=30):: metaname_ds1 = 'metaname for int'
      CHARACTER(LEN=30):: metaname_ds2 = 'metaname for int8'
      CHARACTER(LEN=30):: metaname_ds3 = 'metaname for int16'
      CHARACTER(LEN=30):: metaname_ds4 = 'metaname for int32'
      CHARACTER(LEN=30):: metaname_fds1 = 'metaname for float'
      CHARACTER(LEN=30):: metaname_fds2 = 'metaname for float32'
      CHARACTER(LEN=30):: metaname_fds3 = 'metaname for float64'
      CHARACTER(LEN=30):: metaname_string = 'metaname for string'
      CHARACTER(LEN=20):: meta_pattern(1:4,1:3,1:2), meta_index(1:3,1:2,1:2)
      CHARACTER(LEN=40):: meta_string(1:4,1:3,1:2,1:3,1:2,1:2)
      DATA meta_pattern/ 'm111','m211','m311','m411', 'm121','m221','m321','m421', &
                         'm131','m231','m331','m431', 'm112','m212','m312','m412', &
                         'm122','m222','m322','m422', 'm132','m232','m332','m432' /
      DATA meta_index/ '-111', '-211', '-311', '-121', '-221', '-321', &
                       '-112', '-212', '-312', '-122', '-222', '-322' /
      INTEGER:: mds1(1:4,1:3,1:2,1:3,1:2,1:2)
      INTEGER*1:: mds2(1:4,1:3,1:2,1:3,1:2,1:2)
      INTEGER*2:: mds3(1:4,1:3,1:2,1:3,1:2,1:2)
      INTEGER*4:: mds4(1:4,1:3,1:2,1:3,1:2,1:2)
      REAL:: mfds1(1:4,1:3,1:2,1:3,1:2,1:2)
      REAL*4:: mfds2(1:4,1:3,1:2,1:3,1:2,1:2)
      REAL*8:: mfds3(1:4,1:3,1:2,1:3,1:2,1:2)
      INTEGER:: meta_factor = 1
      REAL:: meta_f_factor = 100.0
      CHARACTER(LEN=40):: meta_str1, meta_str2, meta_str3

!  Set latitude & longitude data
!
      CHARACTER(LEN=20):: lat_label, lon_label
      DATA lat_label/'latitude'/
      DATA lon_label/'longitude'/
      INTEGER:: kind
      DATA kind/3/
      INTEGER:: start_scan, end_scan
      DATA start_scan/10/
      DATA end_scan/15/
      INTEGER,PARAMETER:: num_scan = 6
      REAL:: ds_latlon(1:2,1:num_scan)
      DATA ds_latlon/30.,130., 32.,132., 32.,134., 33.,134., 33.,133., 32.,132. /

!  Reserve Input data Storing Area
!
      INTEGER,PARAMETER:: mmrank = 6
      INTEGER:: dims_get(1:mmrank)
      INTEGER:: nnrank
      INTEGER,PARAMETER:: total = 300
      INTEGER:: element, total_get, total_line

!  Get Area of Dataset Values
      INTEGER:: ds1_get(1:total)
      INTEGER*1:: ds2_get(1:total)
      INTEGER*2:: ds3_get(1:total)
      INTEGER*4:: ds4_get(1:total)
      REAL:: fds1_get(1:total)
      REAL*4:: fds2_get(1:total)
      REAL*8:: fds3_get(1:total)
      REAL:: ds_latlon_get(1:2*num_scan)

!  Get Area of meta Values
      INTEGER,PARAMETER:: mtotal = 300
      CHARACTER(LEN=50)::meta_string_get(1:total)
      INTEGER:: mds1_get(1:total)
      INTEGER*1:: mds2_get(1:total)
      INTEGER*2:: mds3_get(1:total)
      INTEGER*4:: mds4_get(1:total)
      REAL:: mfds1_get(1:total)
      REAL*4:: mfds2_get(1:total)
      REAL*8:: mfds3_get(1:total)

!  Declare Working Variables
      INTEGER:: i,j,k,ii,jj,kk,i1,i2
      INTEGER:: ret


!--------------------------------------------------------------------
!  Setup Environment
!--------------------------------------------------------------------

!  Build Dataset data
      DO k = 1,2
      DO j = 1,2
      DO i = 1,3
          factor = ds_factor(i,j,k)
          DO kk = 1,2
          DO jj = 1,3
          DO ii = 1,4
              ds1(ii,jj,kk,i,j,k ) = ds1_pattern(ii,jj,kk) / factor
              ds2(ii,jj,kk,i,j,k ) = ds2_pattern(ii,jj,kk) / factor
              ds3(ii,jj,kk,i,j,k ) = ds3_pattern(ii,jj,kk) / factor
              ds4(ii,jj,kk,i,j,k ) = ds4_pattern(ii,jj,kk) / factor
          END DO
          END DO
          END DO
      END DO
      END DO
      END DO

      DO k = 1,2
      DO j = 1,2
      DO i = 1,3
          f_factor = fds_factor(i,j,k)
          DO kk = 1,2
          DO jj = 1,3
          DO ii = 1,4
              fds1(ii,jj,kk,i,j,k ) = fds1_pattern(ii,jj,kk) * f_factor
              fds2(ii,jj,kk,i,j,k ) = fds2_pattern(ii,jj,kk) * f_factor
              fds3(ii,jj,kk,i,j,k ) = fds3_pattern(ii,jj,kk) * f_factor
          END DO
          END DO
          END DO
      END DO
      END DO
      END DO

!  Build Meta data
      DO k = 1,2
      DO j = 1,2
      DO i = 1,3
          meta_str3 = meta_index(i,j,k)
          DO kk = 1,2
          DO jj = 1,3
          DO ii = 1,4
              meta_str2 = meta_pattern(ii,jj,kk)
              meta_str1 = TRIM(meta_str2) // TRIM(meta_str3)
              meta_string(ii,jj,kk,i,j,k ) = meta_str1
          END DO
          END DO
          END DO
      END DO
      END DO
      END DO

      DO k = 1,2
      DO j = 1,2
      DO i = 1,3
          DO kk = 1,2
          DO jj = 1,3
          DO ii = 1,4
              mds1(ii,jj,kk,i,j,k ) = ds1_pattern(ii,jj,kk) / meta_factor
              mds2(ii,jj,kk,i,j,k ) = ds2_pattern(ii,jj,kk) / meta_factor
              mds3(ii,jj,kk,i,j,k ) = ds3_pattern(ii,jj,kk) / meta_factor
              mds4(ii,jj,kk,i,j,k ) = ds4_pattern(ii,jj,kk) / meta_factor
          END DO
          END DO
          END DO
      END DO
      END DO
      END DO

      DO k = 1,2
      DO j = 1,2
      DO i = 1,3
          DO kk = 1,2
          DO jj = 1,3
          DO ii = 1,4
              mfds1(ii,jj,kk,i,j,k ) = fds1_pattern(ii,jj,kk) * meta_f_factor
              mfds2(ii,jj,kk,i,j,k ) = fds2_pattern(ii,jj,kk) * meta_f_factor
              mfds3(ii,jj,kk,i,j,k ) = fds3_pattern(ii,jj,kk) * meta_f_factor
          END DO
          END DO
          END DO
      END DO
      END DO
      END DO

      PRINT *, '***** START OF sample.f95 *****'

!  load config file
!
      ret = SGTK_load_config( configfile )
      IF( ret < 0 ) THEN
          PRINT *, 'config file loading  error. program stopped.',ret
!          STOP
      END IF
      

!--------------------------------------------------------------------
!  Output product
!--------------------------------------------------------------------

!  Create a HDF5file
!
      hdf5_id = SGTK_open_file( fname )
      IF( hdf5_id < 0 ) THEN
          PRINT *, 'HDF5file open error. program stopped.',ret
!          STOP
      END IF

!  make a Group
!
      ret = SGTK_make_group( hdf5_id, 'sample group')
      IF( ret < 0 ) THEN
          PRINT *, 'error in making group.',ret
!          STOP
      END IF

!  get UTC and convert into TAI
!
      CALL DATE_AND_TIME( date0_now, time0_now, zone0_now, datetime_values )
      PRINT *, date0_now, time0_now, zone0_now
      year_now    = datetime_values(1)
      month_now   = datetime_values(2)
      day_now     = datetime_values(3)
      hour_now    = datetime_values(5)
      minutes_now = datetime_values(6)
      second_now  = datetime_values(7)
      mills_now   = datetime_values(8)
      ret = SGTK_utc_to_tai93( year_now, month_now, day_now, &
            hour_now, minutes_now, second_now, mills_now, tai93_value )
      IF( ret < 0 ) THEN
          PRINT *, 'to TAI93 convert error.',ret
!          STOP
      END IF
      ret = SGTK_tai93_to_utc( tai93_value, year_now, month_now, day_now, &
            hour_now, minutes_now, second_now, mills_now )
      IF( ret < 0 ) THEN
          PRINT *, 'to TAI93 convert error.',ret
!          STOP
      END IF
      PRINT *, 'date and time from TAI93 = ', &
            year_now, month_now, day_now, hour_now, minutes_now, second_now, mills_now

!  get global data of latitude and longitude
!
      ret = SGTK_get_global_latlon( granule_id, latlon_get )
      IF( ret < 0 ) THEN
          PRINT *, 'global latitude and longitude getting  error.',ret
!          STOP
      END IF
      PRINT *, 'No.  latitude   longitude'
      DO j = 1, ret
          k = 2*(j - 1) + 1
          WRITE(*,'(I3  F10.3 F10.3)') j, latlon_get(k), latlon_get(k+1)
      END DO

!  creat a dataset by setting the dimension sizes
!     
      ret = SGTK_set_dims( hdf5_id, access_label, dimsizes, rank, dataset_type, dimlabels )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset creation error.',ret
!          STOP
      END IF

!  put integer type data into the dataset
!
      ret = SGTK_set_dataset_int( hdf5_id, access_label, dimsizes, rank, ds1 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(integer type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_int8( hdf5_id, access_label, dimsizes, rank, ds2 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(integer*1 type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_int16( hdf5_id, access_label, dimsizes, rank, ds3 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(integer*2 type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_int32( hdf5_id, access_label, dimsizes, rank, ds4 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(integer*4 type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_uint( hdf5_id, access_label, dimsizes, rank, ds1 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(integer type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_uint8( hdf5_id, access_label, dimsizes, rank, ds2 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(integer*1 type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_uint16( hdf5_id, access_label, dimsizes, rank, ds3 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(integer*2 type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_uint32( hdf5_id, access_label, dimsizes, rank, ds4 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(integer*4 type) error.',ret
!          STOP
      END IF

!  put float type data into the dataset
!
      ret = SGTK_set_dataset_float( hdf5_id, access_label, value_kind, dimsizes, rank, fds1 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(float type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_float32( hdf5_id, access_label, value_kind, dimsizes, rank, fds2 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(float32 type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_float64( hdf5_id, access_label, value_kind, dimsizes, rank, fds3 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put(float64 type) error.',ret
!          STOP
      END IF
      ret = SGTK_set_dataset_latlon( hdf5_id, lat_label, lon_label, kind, &
                                     start_scan, end_scan, num_scan, ds_latlon )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset put((latitude longitude) type) error.',ret
!          STOP
      END IF

!  Put meta data( string type ) into the dataset
!
      ret = SGTK_set_meta_string( hdf5_id, access_label, meta_name, &
                                  meta_string )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(string type) put error.',ret
!          STOP
      END IF

!  Put meta data( integer type ) into the dataset
!
      ret = SGTK_set_meta_int( hdf5_id, access_label, metaname_ds1, &
                               mds1 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(int type) put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_int8( hdf5_id, access_label, metaname_ds2, &
                               mds2 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(int8 type) put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_int16( hdf5_id, access_label, metaname_ds3, &
                               mds3 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(int16 type) put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_int32( hdf5_id, access_label, metaname_ds4, &
                               mds4 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(int32 type) put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_uint( hdf5_id, access_label, metaname_ds1, &
                               mds1 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta(uint type) data put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_uint8( hdf5_id, access_label, metaname_ds2, &
                               mds2 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(uint8 type) put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_uint16( hdf5_id, access_label, metaname_ds3, &
                               mds3 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(uint16 type) put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_uint32( hdf5_id, access_label, metaname_ds4, &
                               mds4 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(uint32 type) put error.',ret
!          STOP
      END IF

!  put meta data(float type) into the dataset
!
      ret = SGTK_set_meta_float( hdf5_id, access_label, metaname_fds1, mfds1 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(float type) put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_float32( hdf5_id, access_label, metaname_fds2, mfds2 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(float32 type) put error.',ret
!          STOP
      END IF
      ret = SGTK_set_meta_float64( hdf5_id, access_label, metaname_fds3, mfds3 )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file meta data(float64 type) put error.',ret
!          STOP
      END IF

!  Close the HDF5file
!
      ret = SGTK_close_file( hdf5_id )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file close error.',ret
!          STOP
      END IF

!--------------------------------------------------------------------
!  Input product
!--------------------------------------------------------------------

!  Open the HDF5file
!
      hdf5_id = SGTK_open_file( fname )
      IF( hdf5_id < 0 ) THEN
          PRINT *, 'HDF5file open error. program stopped.',ret
!          STOP
      END IF

!  Get the dimension sizes of the dataset concerned
!     
      ret = SGTK_get_dims( hdf5_id, access_label, dims_get, mmrank )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset form get error.',ret
!          STOP
      END IF
      nnrank = ret
      WRITE(*,'("dataset dimension sizes = ",6(i3, 1x))') dims_get(1:nnrank)
      element = dims_get(1)
      total_get = 1
      total_line = 1
      DO j = 1 , nnrank
!          total_get = total_get * dims_get(j)
          total_get = total_get * dimsizes(j)
      END DO
      IF( total_get > 6) THEN
          total_get = 6
      END IF
      PRINT *,'   element.',element
      IF( element > 0) THEN
          total_line = (total_get + element - 1) / element
      END IF

!  get integer type data from the dataset
!
      DO k = 1,2
          IF( k == 1 ) THEN
		      ret = SGTK_get_dataset_int( hdf5_id, access_label, ds1_get )
		      WRITE(*,'("SGTK_get_dataset_int : ret = ",i3)') ret
		  ELSE
		      ret = SGTK_get_dataset_uint( hdf5_id, access_label, ds1_get )
		      WRITE(*,'("SGTK_get_dataset_uint : ret = ",i3)') ret
          END IF
	      IF( ret < 0 ) THEN
	          PRINT *, 'HDF5file dataset(integer type) get error.',ret
!	          STOP
	      END IF
	      DO j = 1 , total_line
	          i1 = element * (j - 1) + 1
	          i2 = element * j
	          IF( i2 > total_get ) THEN
	              i2 = total_get
	          END IF
	          WRITE(*,'(1X,4(i10,1x))') ds1_get(i1:i2)
	      END DO

          IF( k == 1 ) THEN
		      ret = SGTK_get_dataset_int8( hdf5_id, access_label, ds2_get )
		      WRITE(*,'("SGTK_get_dataset_int8 : ret = ",i3)') ret
          ELSE
		      ret = SGTK_get_dataset_uint8( hdf5_id, access_label, ds2_get )
		      WRITE(*,'("SGTK_get_dataset_uint8 : ret = ",i3)') ret
          END IF
	      IF( ret < 0 ) THEN
	          PRINT *, 'HDF5file dataset(integer*1 type) get error.',ret
!	          STOP
	      END IF
	      DO j = 1 , total_line
	          i1 = element * (j - 1) + 1
	          i2 = element * j
	          IF( i2 > total_get ) THEN
	              i2 = total_get
	          END IF
	          WRITE(*,'(1X,4(i10,1x))') ds2_get(i1:i2)
	      END DO

          IF( k == 1 ) THEN
		      ret = SGTK_get_dataset_int16( hdf5_id, access_label, ds3_get )
		      WRITE(*,'("SGTK_get_dataset_int16 : ret = ",i3)') ret
          ELSE
		      ret = SGTK_get_dataset_uint16( hdf5_id, access_label, ds3_get )
		      WRITE(*,'("SGTK_get_dataset_uint16 : ret = ",i3)') ret
          END IF
	      IF( ret < 0 ) THEN
	          PRINT *, 'HDF5file dataset get(integer*2 type) get error.',ret
!	          STOP
	      END IF
	      DO j = 1 , total_line
	          i1 = element * (j - 1) + 1
	          i2 = element * j
	          IF( i2 > total_get ) THEN
	              i2 = total_get
	          END IF
	          WRITE(*,'(1X,4(i10,1x))') ds3_get(i1:i2)
	      END DO
          IF( k == 1 ) THEN
		      ret = SGTK_get_dataset_int32( hdf5_id, access_label, ds4_get )
		      WRITE(*,'("SGTK_get_dataset_int32 : ret = ",i3)') ret
          ELSE
		      ret = SGTK_get_dataset_uint32( hdf5_id, access_label, ds4_get )
		      WRITE(*,'("SGTK_get_dataset_uint32 : ret = ",i3)') ret
          END IF
	      IF( ret < 0 ) THEN
	          PRINT *, 'HDF5file dataset(integer*4 type) get error.',ret
!	          STOP
	      END IF
	      DO j = 1 , total_line
	          i1 = element * (j - 1) + 1
	          i2 = element * j
	          IF( i2 > total_get ) THEN
	              i2 = total_get
	          END IF
	          WRITE(*,'(1X,4(i10,1x))') ds4_get(i1:i2)
	      END DO
      END DO

!  get float type data from the dataset
!
      ret = SGTK_get_dataset_float( hdf5_id, access_label, value_kind, fds1_get )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset get(float type) get error.',ret
!          STOP
      END IF
      DO j = 1 , total_line
          i1 = element * (j - 1) + 1
          i2 = element * j
          IF( i2 > total_get ) THEN
              i2 = total_get
          END IF
          WRITE(*,'(1X,4(F10.3,1x))') fds1_get(i1:i2)
      END DO
      ret = SGTK_get_dataset_float32( hdf5_id, access_label, value_kind, fds2_get )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset(float32 type) get error.',ret
!          STOP
      END IF
      DO j = 1 , total_line
          i1 = element * (j - 1) + 1
          i2 = element * j
          IF( i2 > total_get ) THEN
              i2 = total_get
          END IF
          WRITE(*,'(1X,4(F10.3,1x))') fds2_get(i1:i2)
      END DO
      ret = SGTK_get_dataset_float64( hdf5_id, access_label, value_kind, fds3_get )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset(float64 type) get error.',ret
!          STOP
      END IF
      DO j = 1 , total_line
          i1 = element * (j - 1) + 1
          i2 = element * j
          IF( i2 > total_get ) THEN
              i2 = total_get
          END IF
          WRITE(*,'(1X,4(F10.3,1x))') fds3_get(i1:i2)
      END DO
      PRINT *, '  ==>hdf5_id=',hdf5_id
      ret = SGTK_get_dataset_latlon( hdf5_id, lat_label, lon_label, kind, &
                                     start_scan, end_scan, ds_latlon_get )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset ((latitude longitude) type) get error.',ret
!          STOP
      END IF
      DO j = 1, ret
          i1 = 2 * (j - 1) + 1
          i2 = 2 * j
          WRITE(*,'(1X,"no. (latitude longitude) -> ",I3,1x,2(F10.3,1x))') j, ds_latlon_get(i1:i2)
      END DO
      
!  Get meta data( string type ) from the dataset
!
      PRINT *, '  ==>hdf5_id=',hdf5_id
      ret = SGTK_get_meta_string( hdf5_id, access_label, metaname_string, meta_string_get )
	  IF( ret < 0 ) THEN
	      PRINT *, 'HDF5file meta data(string type) get error.',ret
!	      STOP
	  END IF
      WRITE(*,'("meta data string value = ",A )') meta_string_get(1:1)
!  Get meta data( integer type ) from the dataset
!
      DO k = 1, 2
          IF( k == 1 ) THEN
		      ret = SGTK_get_meta_int( hdf5_id, access_label, metaname_ds1, &
		                               mds1_get )
		      WRITE(*,'("SGTK_get_meta_int : name ret = ",A,I3)') metaname_ds1,ret
          ELSE
		      ret = SGTK_get_meta_uint( hdf5_id, access_label, metaname_ds1, &
		                               mds1_get )
		      WRITE(*,'("SGTK_get_meta_uint : name ret = ",A,I3)') metaname_ds1,ret
          END IF
          IF( ret < 0 ) THEN
              PRINT *, 'HDF5file meta data(int type) get error.',ret
!              STOP
          END IF
	      DO j = 1 , total_line
	          i1 = element * (j - 1) + 1
	          i2 = element * j
	          IF( i2 > total_get ) THEN
	              i2 = total_get
	          END IF
	          WRITE(*,'(1X,4(i10,1x))') mds1_get(i1:i2)
	      END DO
          IF( k == 1 ) THEN
		      ret = SGTK_get_meta_int8( hdf5_id, access_label, metaname_ds2, &
		                               mds2_get )
		      WRITE(*,'("SGTK_get_meta_int8 : name ret = ",A,I3)') metaname_ds2,ret
          ELSE
		      ret = SGTK_get_meta_uint8( hdf5_id, access_label, metaname_ds2, &
		                               mds2_get )
		      WRITE(*,'("SGTK_get_meta_uint8 : name ret = ",A,I3)') metaname_ds2,ret
          END IF
          IF( ret < 0 ) THEN
              PRINT *, 'HDF5file meta data(int8 type) get error.',ret
!              STOP
          END IF
	      DO j = 1 , total_line
	          i1 = element * (j - 1) + 1
	          i2 = element * j
	          IF( i2 > total_get ) THEN
	              i2 = total_get
	          END IF
	          WRITE(*,'(1X,4(i10,1x))') mds2_get(i1:i2)
	      END DO
          IF( k == 1 ) THEN
		      ret = SGTK_get_meta_int16( hdf5_id, access_label, metaname_ds3, &
		                               mds3_get )
		      WRITE(*,'("SGTK_get_meta_int16 : name ret = ",A,I3)') metaname_ds3,ret
          ELSE
		      ret = SGTK_get_meta_uint16( hdf5_id, access_label, metaname_ds3, &
		                               mds3_get )
		      WRITE(*,'("SGTK_get_meta_uint16 : name ret = ",A,I3)') metaname_ds3,ret
          END IF
          IF( ret < 0 ) THEN
              PRINT *, 'HDF5file meta data(int16 type) get error.',ret
!              STOP
          END IF
	      DO j = 1 , total_line
	          i1 = element * (j - 1) + 1
	          i2 = element * j
	          IF( i2 > total_get ) THEN
	              i2 = total_get
	          END IF
	          WRITE(*,'(1X,4(i10,1x))') mds3_get(i1:i2)
	      END DO
          IF( k == 1 ) THEN
		      ret = SGTK_get_meta_int32( hdf5_id, access_label, metaname_ds4, &
		                               mds4_get )
		      WRITE(*,'("SGTK_get_meta_int32 : name ret = ",A,I3)') metaname_ds4,ret
          ELSE
		      ret = SGTK_get_meta_uint32( hdf5_id, access_label, metaname_ds4, &
		                               mds4_get )
		      WRITE(*,'("SGTK_get_meta_uint32 : name ret = ",A,I3)') metaname_ds4,ret
          END IF
          IF( ret < 0 ) THEN
              PRINT *, 'HDF5file meta data(int32 type) get error.',ret
!              STOP
          END IF
	      DO j = 1 , total_line
	          i1 = element * (j - 1) + 1
	          i2 = element * j
	          IF( i2 > total_get ) THEN
	              i2 = total_get
	          END IF
	          WRITE(*,'(1X,4(i10,1x))') mds4_get(i1:i2)
	      END DO
      END DO

!  get meta data(float type) from the dataset
!
      ret = SGTK_get_meta_float( hdf5_id, access_label, metaname_fds1, fds1_get )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset get(float type) get error.',ret
!          STOP
      END IF
      DO j = 1 , total_line
          i1 = element * (j - 1) + 1
          i2 = element * j
          IF( i2 > total_get ) THEN
              i2 = total_get
          END IF
          WRITE(*,'(1X,4(F10.3,1x))') fds1_get(i1:i2)
      END DO
      ret = SGTK_get_meta_float32( hdf5_id, access_label, metaname_fds2, fds2_get )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset(float32 type) get error.',ret
!          STOP
      END IF
      DO j = 1 , total_line
          i1 = element * (j - 1) + 1
          i2 = element * j
          IF( i2 > total_get ) THEN
              i2 = total_get
          END IF
          WRITE(*,'(1X,4(F10.3,1x))') fds2_get(i1:i2)
      END DO
      ret = SGTK_get_meta_float64( hdf5_id, access_label, metaname_fds3, fds3_get )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file dataset(float64 type) get error.',ret
!          STOP
      END IF
      DO j = 1 , total_line
          i1 = element * (j - 1) + 1
          i2 = element * j
          IF( i2 > total_get ) THEN
              i2 = total_get
          END IF
          WRITE(*,'(1X,4(F10.3,1x))') fds3_get(i1:i2)
      END DO

!  Close the HDF5file
!
      ret = SGTK_close_file( hdf5_id )
      IF( ret < 0 ) THEN
          PRINT *, 'HDF5file close error.',ret
!          STOP
      END IF

      PRINT *, '***** END OF sample.f95 *****'
      END PROGRAM sample
      