#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "SGTK.h"
#include "SGTK_ret_value.h"
//#include "SGTK_ACCESS_LABEL_L1A_VNR_NP.h"

int main(int argc, char *argv[]) {

	int pata_ret = 0;
	
    herr_t status = SUCCESS;
	
	/** make parameter table.
	*/
	printf("SGTK_make_parameter_table : make parameter table\n");
	pata_ret = SGTK_make_parameter_table("./L1/SGTK_ACCESS_LABEL_L1A_VNR_NP");
	printf("pata_ret=%d\n", pata_ret);
	printf("map_size=%ld\n", SGLI_ACCESS_LABEL_TABLE.size());
	
    /** load configulation file.
     */
	printf("SGTK_load_config : load configulation file\n");
	status = SGTK_load_config((char*)"./config.txt");
	if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** open product file.
     */ 
	printf("SGTK_open_file : open product file\n");
	int file_id = SGTK_open_file((char*)"./sample_product.h5");

    /** set dimensions.
     */
	printf("SGTK_set_dims : set dimensions\n");
	int dims[3] = { 3, 11, 7400 };
    char *label[] = { (char*)"lens", (char*)"ch", (char*)"line" ,(char*)"" ,(char*)"" ,(char*)""};
    status = SGTK_set_dims(file_id, (char*)"GC1_L1A_NP_DTS_RAW_DATA_VN01", (int*)&dims, 3, SGTK_H5T_STD_U32_TYPE, label); 
	if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** allocate memory.
     */
	printf("SGTK_malloc : allocate memory to set data\n");
	int *outvalue = (int*)SGTK_malloc(dims, 3, sizeof(int));
	if(outvalue==NULL) {
        printf("[%s][%05d]\n", __FILE__, __LINE__);
        return FAIL;
    }

    /** set dataset value.
     */
	printf("SGTK_set_dataset_int : set dataset value\n");
	for(int i=0; i<3*11*7400; i++) {
        *(outvalue+i) = i;
    }
    status = SGTK_set_dataset_int(file_id, (char*)"GC1_L1A_NP_DTS_RAW_DATA_VN01", dims, 3, outvalue);
	//status = SGTK_set_dataset_int(file_id, (char*)GC1_DTS_RAW_DATA_VN01, dims, 3, outvalue);
    if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** set string attribute value.
     */
	printf("SGTK_set_meta_string : set string attribute value\n");
	char meta_out_string[] = "Count";
    status = SGTK_set_meta_string(file_id, (char*)"GC1_L1A_NP_DTS_RAW_DATA_VN01", (char*)"GC1_L1A_NP_ATR_UNIT", (char*)meta_out_string);
	if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** set numeric attribute value.
     */
	printf("SGTK_set_meta_float : set numeric attribute value\n");
	float meta_float_value = 250.0;
    status = SGTK_set_meta_float(file_id, (char*)"GC1_L1A_NP_DTS_RAW_DATA_VN01", (char*)"GC1_L1A_NP_ATR_SPATIAL_RESOLUTION", meta_float_value);
	if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** allocate memory.
     */
	printf("SGTK_malloc : allocate memory to get data\n");
	int *invalue = (int*)SGTK_malloc(dims, 3, sizeof(int));
	if(outvalue==NULL) {
        printf("[%s][%05d]\n", __FILE__, __LINE__);
        return FAIL;
    }

    /** get dataset value.
     */
	printf("SGTK_get_dataset_int : get dataset value\n");
	status = SGTK_get_dataset_int(file_id, (char*)"GC1_L1A_NP_DTS_RAW_DATA_VN01", &invalue);
	if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** get string attribute value.
     */
	printf("SGTK_get_meta_string : get string attribute value\n");
	char meta_in_string[16];
    memset(meta_in_string, 0, sizeof(meta_in_string));
    status = SGTK_get_meta_string(file_id, (char*)"GC1_L1A_NP_DTS_RAW_DATA_VN01", (char*)"GC1_L1A_NP_ATR_UNIT", (char*)meta_in_string);
	if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** get numeric attribute value.
     */
	printf("SGTK_get_meta_float : get numeric attribute value\n");
	meta_float_value = 0.0;
    status = SGTK_get_meta_float(file_id, (char*)"GC1_L1A_NP_DTS_RAW_DATA_VN01", (char*)"GC1_L1A_NP_ATR_SPATIAL_RESOLUTION", (float*)&meta_float_value);
	if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** close product file.
     */
	printf("SGTK_close_file : close product file\n");
	status = SGTK_close_file(file_id);
	if(SUCCESS!=status) {
        printf("[%s][%05d] status:[%d]\n", __FILE__, __LINE__, status);
        return status;
    }

    /** release alloc'd memory.
     */ 
	printf("SGTK_free : release alloc'd memory\n");
	SGTK_free(outvalue);
    SGTK_free(invalue);

	printf("sample program normal end\n");
	return status;
}
