/*****************************************************************************/
/*  Copyright (C) NEC Corporation 2015 All rights reserved.                  */
/*  File Name    : SGTK_cnf.cpp                                              */
/*  Abstract     : I/O toolkit config file  function                         */
/*  Function List                                                            */
/*      load_config()                : SGTK Config file read funtion         */
/*      analys_cnf_file_rec()        : Config Item Key analys                */
/*      set_vector()                 : Config vector<string> set             */
/*                                                                           */
/*  Author     NEC                                                           */
/*  Date       2015/03/20                                                    */
/*  Version    1.00                                                          */
/*  Notice                                                                   */
/*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#define __SGTK_CNF_MAIN__
#include "SGTK_cnf.h"
#include "SGTK_ret_value.h"
#include "hdf5_hl.h"

static bool analys_cnf_file_rec(const char* rbuf, int* case_flgs, char** top);
static void set_vector(const char* rbur, std::vector<std::string> &vec_buf);
static bool exist_attributes(int id_file, const std::string &path, const std::string &name);

enum conf_lbl_no {
    enum_DATATYPE_COMPRESS_TYPE = 0,
    enum_DATATYPE_COMPRESS_LEVEL,
    enum_DATATYPE_CHUNK,
    enum_DATATYPE_BYTE_ORDER,
    enum_LEAP_SECOND_FILE_PATH,
    enum_EQR_LATLON_FILE_PATH,
    enum_EQA_LATLON_FILE_PATH,
    enum_PS_LATLON_FILE_PATH,
    enum_CORRECT_SLOPE_ATTRIBUTE_NAME,
    enum_CORRECT_OFFSET_ATTRIBUTE_NAME,
    enum_CORRECT_SLOPE_REF_ATTRIBUTE_NAME,
    enum_CORRECT_OFFSET_REF_ATTRIBUTE_NAME,
    enum_CORRECT_BASE_ATTRIBUTE_NAME,
    enum_RANGE_MIN_ATTRIBUTE_NAME,
    enum_RANGE_MAX_ATTRIBUTE_NAME,
    enum_RANGE_ERROR_VALUE,
    enum_max
};

// Config file key define
#define KEY_DATATYPE_COMPRESS_TYPE              "DATATYPE_COMPRESS_TYPE"
#define KEY_DATATYPE_COMPRESS_LEVEL             "DATATYPE_COMPRESS_LEVEL"
#define KEY_DATATYPE_CHUNK                      "DATATYPE_CHUNK"
#define KEY_DATATYPE_BYTE_ORDER                 "DATATYPE_BYTE_ORDER"
#define KEY_LEAP_SECOND_FILE_PATH               "LEAP_SECOND_FILE_PATH"
#define KEY_EQR_LATLON_FILE_PATH                "EQR_LATLON_FILE_PATH"
#define KEY_EQA_LATLON_FILE_PATH                "EQA_LATLON_FILE_PATH"
#define KEY_PS_LATLON_FILE_PATH                 "PS_LATLON_FILE_PATH"
#define KEY_CORRECT_SLOPE_ATTRIBUTE_NAME        "CORRECT_SLOPE_ATTRIBUTE_NAME"
#define KEY_CORRECT_OFFSET_ATTRIBUTE_NAME       "CORRECT_OFFSET_ATTRIBUTE_NAME"
#define KEY_CORRECT_SLOPE_REF_ATTRIBUTE_NAME    "CORRECT_SLOPE_REF_ATTRIBUTE_NAME"
#define KEY_CORRECT_OFFSET_REF_ATTRIBUTE_NAME   "CORRECT_OFFSET_REF_ATTRIBUTE_NAME"
#define KEY_CORRECT_BASE_ATTRIBUTE_NAME         "CORRECT_BASE_ATTRIBUTE_NAME"
#define KEY_RANGE_MIN_ATTRIBUTE_NAME            "RANGE_MIN_ATTRIBUTE_NAME"
#define KEY_RANGE_MAX_ATTRIBUTE_NAME            "RANGE_MAX_ATTRIBUTE_NAME"
#define KEY_RANGE_ERROR_VALUE                   "RANGE_ERROR_VALUE"

struct attr_name_tbl {
//  char            *name;
    std::string     name;
    conf_lbl_no     conf_no;
};

/** load SGTK configuration file.
 *
 * @param   cnf_file_name   [char*][in] SGTK configuration file name.
 * @return result status.
 */
int load_config(const char* cnf_file_name)
{
    FILE    *fp = fopen(cnf_file_name, "r");
    //char    rbuf[1024 * 1024];
    char    rbuf[256]; // 20150314
    int     ret = SUCCESS;

    // File Oepn
    if (fp == NULL)
    {
        return ERROR_FILE_OPEN;
    }

    // Config file read
    while (fgets(rbuf, sizeof(rbuf), fp) != NULL)
    {
        char    *top = NULL;
        int     case_flg = 0;
        int     level    = 0;
        int     chunk    = 0;

        rbuf[strlen(rbuf)-1] = '\0';

        if (analys_cnf_file_rec(rbuf, &case_flg, &top) != true)
        {
            ret = ERROR_CONFIG_ANALYS;  // Key Error
            break;
        }

        switch (case_flg) {
        case enum_DATATYPE_COMPRESS_TYPE:
            if (strcmp(top, "gzip") == 0)
            {
                conf_obj.compress_type = true;
            }
            else if (strlen(top) == 0) {
                conf_obj.compress_type = false;
            }
            else
            {
                ret = ERROR_CONFIG_CMP_TYPE;    // DATATYPE_COMPRESS_TYPE error
            }
            break;
        case enum_DATATYPE_COMPRESS_LEVEL:
            level = atoi(top);
            if ((level < 1) || (level > 9))
            {
                ret = ERROR_CONFIG_CMP_LVL;     // DATATYPE_COMPRESS_LEVEL error
            }
            conf_obj.compress_level = level;
            break;
        case enum_DATATYPE_CHUNK:
            chunk = atoi(top);
            conf_obj.chunk = chunk;
            break;
        case enum_DATATYPE_BYTE_ORDER:
            if (strcmp(top, "LE") == 0)
            {
                conf_obj.byte_order = true;
            }
            else if (strcmp(top, "BE") == 0)
            {
                conf_obj.byte_order = false;
            }
            else
            {
                ret = ERROR_CONFIG_BYTE_ORDR;   // DATATYPE_BYTE_ORDER error
            }
            break;
        case enum_LEAP_SECOND_FILE_PATH:
            conf_obj.leap_sec_file_path = top;
            break;
        case enum_EQR_LATLON_FILE_PATH:
            conf_obj.eqr_latlon_file_path = top;
            break;
        case enum_EQA_LATLON_FILE_PATH:
            conf_obj.eqa_latlon_file_path = top;
            break;
        case enum_PS_LATLON_FILE_PATH:
            conf_obj.ps_latlon_file_path = top;
            break;
        case enum_CORRECT_SLOPE_ATTRIBUTE_NAME:
            set_vector(top, conf_obj.correct_slope_attr_name);
            break;
        case enum_CORRECT_OFFSET_ATTRIBUTE_NAME:
            set_vector(top, conf_obj.correct_offset_attr_name);
            break;
        case enum_CORRECT_SLOPE_REF_ATTRIBUTE_NAME:
            set_vector(top, conf_obj.correct_slope_ref_attr_name);
            break;
        case enum_CORRECT_OFFSET_REF_ATTRIBUTE_NAME:
            set_vector(top, conf_obj.correct_offset_ref_attr_name);
            break;
        case enum_CORRECT_BASE_ATTRIBUTE_NAME:
            set_vector(top, conf_obj.correct_base_attr_name);
            break;
        case enum_RANGE_MIN_ATTRIBUTE_NAME:
            set_vector(top, conf_obj.range_min_attr_name);
            break;
        case enum_RANGE_MAX_ATTRIBUTE_NAME:
            set_vector(top, conf_obj.range_max_attr_name);
            break;
        case enum_RANGE_ERROR_VALUE:
            set_vector(top, conf_obj.range_err_value);
            break;
        default:
            fclose(fp);
            ret = ERROR_CONFIG_FILE_KEY;    // Key Error
            break;
        }

        if (ret != SUCCESS)
        {
            break;
        }
    }

    // File Close
    fclose(fp);

    return ret;
}

/** analyze SGTK configuration file records.
 *
 * @param   rbuf        [char*][in]     record data.
 * @param   case_flgs   [int*][out]     type of record data.
 * @param   top         [char**][top]   parameter value.
 * @return result status.
 */
static bool analys_cnf_file_rec(const char* rbuf, int* case_flgs, char** top)
{
    bool            flg = false;
    attr_name_tbl   atrtbl[] = {
        {KEY_DATATYPE_COMPRESS_TYPE,            enum_DATATYPE_COMPRESS_TYPE},
        {KEY_DATATYPE_COMPRESS_LEVEL,           enum_DATATYPE_COMPRESS_LEVEL},
        {KEY_DATATYPE_CHUNK,                    enum_DATATYPE_CHUNK},
        {KEY_DATATYPE_BYTE_ORDER,               enum_DATATYPE_BYTE_ORDER},
        {KEY_LEAP_SECOND_FILE_PATH,             enum_LEAP_SECOND_FILE_PATH},
        {KEY_EQR_LATLON_FILE_PATH,              enum_EQR_LATLON_FILE_PATH},
        {KEY_EQA_LATLON_FILE_PATH,              enum_EQA_LATLON_FILE_PATH},
        {KEY_PS_LATLON_FILE_PATH,               enum_PS_LATLON_FILE_PATH},
        {KEY_CORRECT_SLOPE_ATTRIBUTE_NAME,      enum_CORRECT_SLOPE_ATTRIBUTE_NAME},
        {KEY_CORRECT_OFFSET_ATTRIBUTE_NAME,     enum_CORRECT_OFFSET_ATTRIBUTE_NAME},
        {KEY_CORRECT_SLOPE_REF_ATTRIBUTE_NAME,  enum_CORRECT_SLOPE_REF_ATTRIBUTE_NAME},
        {KEY_CORRECT_OFFSET_REF_ATTRIBUTE_NAME, enum_CORRECT_OFFSET_REF_ATTRIBUTE_NAME},
        {KEY_CORRECT_BASE_ATTRIBUTE_NAME,       enum_CORRECT_BASE_ATTRIBUTE_NAME},
        {KEY_RANGE_MIN_ATTRIBUTE_NAME,          enum_RANGE_MIN_ATTRIBUTE_NAME},
        {KEY_RANGE_MAX_ATTRIBUTE_NAME,          enum_RANGE_MAX_ATTRIBUTE_NAME},
        {KEY_RANGE_ERROR_VALUE,                 enum_RANGE_ERROR_VALUE}
    };

    for (int count = 0; count < enum_max; ++count)
    {
        if (strncmp(atrtbl[count].name.c_str(), rbuf, strlen(atrtbl[count].name.c_str())) == 0)
        {
            *top = (char *)&rbuf[strlen(atrtbl[count].name.c_str())+1];
            *case_flgs = atrtbl[count].conf_no;
            flg = true;
            break;
        }
    } // End of for

    return flg;
}

/** set vector for multiple value parameters.
 *
 * @param   rbuf    [char*][in]             record data.
 * @param   vec_buf [vector<string>][out]   parameter value vector.
 */
static void set_vector(const char* rbur, std::vector<std::string> &vec_buf)
{
    char    *top;
    char    *tmp;
    char    buf[1024];

    vec_buf.clear();

    for (top = (char*)rbur; ; top = tmp)
    {
        if(0==top) break;
        memset(buf, 0x00, sizeof(buf));
        tmp = strchr(top, ',');
        if (tmp > 0)
        {
            memcpy(buf, top, (int)(tmp - top));
        }
        else
        {
            strcpy(buf, top);
        }

        vec_buf.push_back(buf);
    }

    return;
}

/** check attribute is exists.
 * 
 * @param   id_file     [int][in]       HDF5 file id.
 * @param   path        [string][in]    dataset path.
 * @param   name        [string][in]    attribute name.
 * @return  result status.
 */
static bool exist_attributes(int id_file, const std::string &path, const std::string &name)
{
    bool    ret = true;
    hid_t   id_group = H5Dopen(id_file, path.c_str(), H5P_DEFAULT);
    hid_t   at       = H5Aopen(id_group, name.c_str(), H5P_DEFAULT);

    if (at < 0)
    {
        ret = false;
    }
    else
    {
        H5Aclose(at);
    }
    H5Dclose(id_group);

    return ret;
}

/** 
 *
 * @param   id_file     [int][in]           HDF5 file id.
 * @param   label       [char*][in]         dataset path.
 * @param   label_items [vector<string>]    attibute name vector.
 * @return index of attribute name vector, if exists.
 */
int find_attr_name_data(
                        int                         id_file,
                        char*                       label,
                        std::vector<std::string>    label_items)
{
    int     elment = 0;
    int     max    = label_items.size();

    for ( ; elment < max; elment++)
    {
        bool    ret = exist_attributes(id_file, label, label_items.at(elment));

        if (ret == true)
        {
            return elment;
        }
    }

    return -1;
}
