/************************************************************************/
/* Copyright (c) NEC Corporation 2015 All right reserved                */
/* File Name    :  SGTKfwrap.c                                          */
/* Abstract     :  Fortran Interface to SGTK C modules                  */
/* Author       :  NEC                                                  */
/* Date         :  2015/03/06                                           */
/* Version      :  1.0                                                  */
/************************************************************************/
/***
  Ver  0.1 (specified in 2015.03.09
            and updateded for SGTK_set_dims parameters)
  Ver  0.2 (specified in 2015.03.15)
***/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include "SGTK_wrapper.h"

#define SUCCESS 0
//typedef char int8_t;
typedef short int16_t;
//typedef long int32_t;
typedef unsigned int uint;
typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
//typedef unsigned long uint32_t;

/*--------------------------------------------------------------*/
/*                                                              */
/*  convert fortran's character array into c's string           */
/*  arguments :  char *src_w  -> fortran's charater array       */
/*               int  len_w   -> character array's size         */
/*  return    :  char *     -> address of c's string            */
/*                                                              */
/*--------------------------------------------------------------*/
char *SGTK_convstring_ftoc(char *src_w, int len_w)
{
	int i, j, k;
	int ds;
	char s;
	char *ps;
	
	ds = len_w + 1;
    ps = (char *)malloc( ds );
    
    k = 0;
    for( i = 0; i < ds; i++ )
    {
		j = ds - i - 1 ;
		s = '\0';
		if( i > 0 )
        {
			s = *(src_w + j);
		}
		
		if( k == 0 )
		{
			if( s == ' ' || s == '\0' )
			{
				s = '\0';
			}
			else
			{
				k = 1;
			}
		}
		
		*(ps + j) = s;
	}
	return ps;
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  convert c's string into fortran's character array           */
/*  arguments :  char *src_w  -> c's string area                */
/*               char *dest_w -> fortran's charater array      */
/*               int  dlen_w  -> character array's size         */
/*  return    :  int          -> length of string               */
/*                                                              */
/*--------------------------------------------------------------*/
int SGTK_convstring_ctof(char *src_w, char *dest_w, int dlen_w)
{
	int i, j, k;
//  int ds;
	char s;
//	char *ps;
	
    k = 0;
    i = 0;
	for (j = 0; j < dlen_w; j++ )
    {
		if( k == 0 )
		{
			s = *(src_w + j);
			if( s == '\0' )
			{
				k = 1;
				s = ' ';
			}
			else
			{
				i++;
			}
		}
		else 
		{
			s = ' ';
		}
		*(dest_w + j) = s;
	}
	return i;
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  open HDF5 file and return its handle                        */
/*  arguments :  char *file_name  -> fortran's charater array   */
/*               int  flen        -> character array's size     */
/*  return    :  int    -> opened file's handle                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_open_file_(char *file_name, int *flen)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( file_name, flen );
	ret = SGTK_open_file( ps );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_open_file__(char *file_name, int *flen)
{
	return(sgtk_open_file_( file_name, flen ) );
}
/*--------------------------------------------------------------*/
/*                                                              */
/*  close HDF5 file and return its handle                       */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle  */
/*  return    :  int    -> closed status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_close_file_(int *p_hdf5_file_handle)
{
    return (SGTK_close_file( *p_hdf5_file_handle ));
}

/* fortran77 */
int sgtk_close_file__(int *p_hdf5_file_handle)
{
	printf("sgtk_close_file__  start\n");
    return(sgtk_close_file_( p_hdf5_file_handle ) );
}

/*--------------------------------------------------------------*/
/*                                                              */
/*  create a blank group                                        */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle  */
/*               char *group  -> group name created             */
/*  return    :  int    -> closed status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_make_group_( int *p_hdf5_file_handle, char *group, int glen )
{
	int ret;
	char *ps;
	
	ps = SGTK_convstring_ftoc( group, glen );
	ret = SGTK_make_group( (*p_hdf5_file_handle), ps );
	free( ps );
	return ret;	
}

/* fortran77 */
int sgtk_make_group__( int *p_hdf5_file_handle, char *group, int glen )
{
	return(sgtk_make_group_( p_hdf5_file_handle, group, glen ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  convert time base from UTC to TAI93                         */
/*  arguments :  int *p_year  -> year in UTC                    */
/*               int *p_month -> month in UTC                   */
/*               int *p_day   -> day in UTC                     */
/*               int *p_hour  -> hour in UTC                    */
/*               int *p_min   -> min in UTC                     */
/*               int *p_sec   -> sec in UTC                     */
/*               int *p_msec  -> msec in UTC                    */
/*               double *tai93time -> time in TAI93             */
/*  return    :  int    -> time get status                      */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_utc_to_tai93_(int *p_year, int *p_month, int *p_day, 
                       int *p_hour, int *p_min, int *p_sec, int *p_msec,
                       double *tai93time)
{
    int ret;
    ret = SGTK_utc_to_tai93( (*p_year), (*p_month), (*p_day),
                             (*p_hour), (*p_min), (*p_sec), (*p_msec),
                             tai93time);
    return ret;
}

/* fortran77 */
int sgtk_utc_to_tai93__(int *p_year, int *p_month, int *p_day, 
                       int *p_hour, int *p_min, int *p_sec, int *p_msec,
                       double *tai93time)
{
	return(sgtk_utc_to_tai93_(p_year, p_month, p_day, 
                       p_hour, p_min, p_sec, p_msec,
                       tai93time) );
}
	

/*--------------------------------------------------------------*/
/*                                                              */
/*  convert time base from TAI93 to UTC                         */
/*  arguments :  double *p_tai93time -> time in TAI93           */
/*               int *p_year  -> year in UTC                    */
/*               int *p_month -> month in UTC                   */
/*               int *p_day   -> day in UTC                     */
/*               int *p_hour  -> hour in UTC                    */
/*               int *p_min   -> min in UTC                     */
/*               int *p_sec   -> sec in UTC                     */
/*               int *p_msec  -> msec in UTC                    */
/*  return    :  int    -> time get status                      */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_tai93_to_utc_(double *p_tai93time,
                       int *p_year, int *p_month, int *p_day, 
                       int *p_hour, int *p_min, int *p_sec, int *p_msec)
{
    int ret;
    ret = SGTK_tai93_to_utc( (*p_tai93time),
                             p_year, p_month, p_day,
                             p_hour, p_min, p_sec, p_msec );
    return ret;
}

/* fortran77 */
int sgtk_tai93_to_utc__(double *p_tai93time,
                       int *p_year, int *p_month, int *p_day, 
                       int *p_hour, int *p_min, int *p_sec, int *p_msec)
{
	return(sgtk_tai93_to_utc_(p_tai93time,
                       p_year, p_month, p_day, 
                       p_hour, p_min, p_sec, p_msec) );
}

/*--------------------------------------------------------------*/
/*                                                              */
/*  get global data of lattitudes and longitudes                */
/*  arguments :  char  *granule_id  -> granule ID               */
/*               float *p_latlon    -> acquired data            */
/*  return    :  int         -> the number of acquired data     */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_global_latlon_(char *granule_id, float *p_latlon, int granule_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( granule_id, granule_len );
	ret = SGTK_get_global_latlon( ps, p_latlon );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_global_latlon__(char *granule_id, float *p_latlon, int granule_len)
{
    return(sgtk_get_global_latlon_(granule_id, p_latlon, granule_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data of lattitudes and longitudes                       */
/*  arguments :  char *config_file  -> configuration file's name */
/*  return    :  int         -> data load status                */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_load_config_(char *config_file, int flen)
{
	int ret;
	char *ps;
	
    ps = SGTK_convstring_ftoc( config_file, flen );
	ret = SGTK_load_config( ps );
	free( ps );	
    return ret;
}

/* fortran77 */
int sgtk_load_config__(char *config_file, int flen)
{
    return(sgtk_load_config_(config_file, flen) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get each dimension's size of a dataset                      */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               int *p_dimsize -> dataset dimension size (at most 6 dimension)  */
/*               int *p_maxdims -> maximum number of dimensions */
/*               int label_len  -> length of dataset access label */
/*  return    :  int         -> the number of dimensions        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dims_(int *p_hdf5_file_handle, char *dataset_access_label, int *p_dimsizes, int *p_maxdims, int label_len )
{
	int ret;
	char *ps;
	int mdims;
	int *q;
	int i;
	
	mdims = (*p_maxdims);
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
	ret = SGTK_get_dims( (*p_hdf5_file_handle), ps, &q );
	if( ret > 0 )
	{
	    for( i = 0; i < ret; i++ )
	    {
			if( i < mdims )
			{
			    *(p_dimsizes + i) = *(q + i);
			}
		}
		if( ret > mdims )
		{
			ret = mdims;
		}
		free( q );
    }
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_dims__(int *p_hdf5_file_handle, char *dataset_access_label, int *p_dimsizes, int *p_maxdims, int label_len )
{
	return(sgtk_get_dims_(p_hdf5_file_handle, dataset_access_label, p_dimsizes, p_maxdims, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type int) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               int *p_get_value          -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_int_(int *p_hdf5_file_handle, char *dataset_access_label, int *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_int( (*p_hdf5_file_handle), ps, &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_int__(int *p_hdf5_file_handle, char *dataset_access_label, int *p_get_value, int label_len)
{
	return(sgtk_get_dataset_int_(p_hdf5_file_handle, dataset_access_label, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type int*8) of a dataset                           */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               int8_t *p_get_value       -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_int8_(int *p_hdf5_file_handle, char *dataset_access_label, int8_t *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_int8( (*p_hdf5_file_handle), ps, &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_int8__(int *p_hdf5_file_handle, char *dataset_access_label, int8_t *p_get_value, int label_len)
{
	return(sgtk_get_dataset_int8_(p_hdf5_file_handle, dataset_access_label, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type int*16) of a dataset                          */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               int16_t *p_get_value      -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_int16_(int *p_hdf5_file_handle, char *dataset_access_label, int16_t *p_get_value, int label_len)
{
	int ret;
	char *ps;
	
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_int16( (*p_hdf5_file_handle), ps, &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_int16__(int *p_hdf5_file_handle, char *dataset_access_label, int16_t *p_get_value, int label_len)
{
	return(sgtk_get_dataset_int16_(p_hdf5_file_handle, dataset_access_label, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type int*32) of a dataset                          */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               int32_t *p_get_value      -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_int32_(int *p_hdf5_file_handle, char *dataset_access_label, int32_t *p_get_value, int label_len)
{
	int ret;
	char *ps;
	
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_int32( (*p_hdf5_file_handle), ps, &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_int32__(int *p_hdf5_file_handle, char *dataset_access_label, int32_t *p_get_value, int label_len)
{
	return(sgtk_get_dataset_int32_(p_hdf5_file_handle, dataset_access_label, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type uint) of a dataset                            */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               uint *p_get_value         -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_uint_(int *p_hdf5_file_handle, char *dataset_access_label, uint *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_uint( (*p_hdf5_file_handle), ps, &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_uint__(int *p_hdf5_file_handle, char *dataset_access_label, uint *p_get_value, int label_len)
{
	return(sgtk_get_dataset_uint_(p_hdf5_file_handle, dataset_access_label, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type uint*8) of a dataset                          */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               uint8_t *p_get_value      -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_uint8_(int *p_hdf5_file_handle, char *dataset_access_label, uint8_t *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_uint8( (*p_hdf5_file_handle), ps, &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_uint8__(int *p_hdf5_file_handle, char *dataset_access_label, uint8_t *p_get_value, int label_len)
{
	return(sgtk_get_dataset_uint8_(p_hdf5_file_handle, dataset_access_label, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type uint*16) of a dataset                          */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               uint16_t *p_get_value     -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_uint16_(int *p_hdf5_file_handle, char *dataset_access_label, uint16_t *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_uint16( (*p_hdf5_file_handle), ps, &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_uint16__(int *p_hdf5_file_handle, char *dataset_access_label, uint16_t *p_get_value, int label_len)
{
	return(sgtk_get_dataset_uint16_(p_hdf5_file_handle, dataset_access_label, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type uint*32) of a dataset                          */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               uint32_t *p_get_value     -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_uint32_(int *p_hdf5_file_handle, char *dataset_access_label, uint32_t *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_uint32( (*p_hdf5_file_handle), ps, &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_uint32__(int *p_hdf5_file_handle, char *dataset_access_label, uint32_t *p_get_value, int label_len)
{
	return(sgtk_get_dataset_uint32_(p_hdf5_file_handle, dataset_access_label, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type real) of a dataset                           */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               int *p_kind           -> kind of dataset value */
/*               float *p_get_value        -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_float_(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind, float *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_float( (*p_hdf5_file_handle), ps, (*p_kind), &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_float__(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind, float *p_get_value, int label_len)
{
	return(sgtk_get_dataset_float_(p_hdf5_file_handle, dataset_access_label, p_kind, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type real(4)) of a dataset                         */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               int *p_kind           -> kind of dataset value */
/*               float *p_get_value        -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_float32_(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind, float *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_float32( (*p_hdf5_file_handle), ps, (*p_kind), &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_float32__(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind, float *p_get_value, int label_len)
{
	return(sgtk_get_dataset_float32_(p_hdf5_file_handle, dataset_access_label, p_kind, p_get_value, label_len) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get data(type real(8)) of a dataset                         */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *dataset_access_lbl -> dataset access label */
/*               int *p_kind           -> kind of dataset value */
/*               double *p_get_value       -> data storing area */
/*  return    :  int         -> data get status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_dataset_float64_(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind, double *p_get_value, int label_len)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len);
    ret = SGTK_get_dataset_float64( (*p_hdf5_file_handle), ps, (*p_kind), &(p_get_value) );
    free( ps );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_float64__(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind, double *p_get_value, int label_len)
{
	return(sgtk_get_dataset_float64_(p_hdf5_file_handle, dataset_access_label, p_kind, p_get_value, label_len) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  get scanned data of latitudes and longitudes                    */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *latitude   -> latitude access label          */
/*               char *longitude  -> longitude access label         */
/*               int *p_kind       -> data kind                     */
/*               int *p_start_no   -> get start scan no.            */
/*               int *p_end_no     -> get end scan no.              */
/*               float *p_latlon   -> data storing area             */
/*  return    :  int         -> the number of acquired data         */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_get_dataset_latlon_(int *p_hdf5_file_handle, char *latitude, char *longitude, int *p_kind,
                             int *p_start_no, int *p_end_no, float *p_latlon, int latlabel_len, int lonlabel_len)
{
	int ret;
	char *ps1;
	char *ps2;
	
	ps1 = SGTK_convstring_ftoc( latitude, latlabel_len);
	ps2 = SGTK_convstring_ftoc( longitude, lonlabel_len);
	ret = SGTK_get_dataset_latlon( *(p_hdf5_file_handle), ps1, ps2, *(p_kind),
	                               *(p_start_no), *(p_end_no), &(p_latlon) );
	free( ps1 );
	free( ps2 );
    return ret;
}

/* fortran77 */
int sgtk_get_dataset_latlon__(int *p_hdf5_file_handle, char *latitude, char *longitude, int *p_kind,
                             int *p_start_no, int *p_end_no, float *p_latlon, int latlabel_len, int lonlabel_len)
{
	return(sgtk_get_dataset_latlon_(p_hdf5_file_handle, latitude, longitude, p_kind,
                             p_start_no, p_end_no, p_latlon, latlabel_len, lonlabel_len) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  get meta data(type string)                                      */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label     ->  dataset access label           */
/*               char *meta      ->  meta data name                 */
/*               char *p_value   ->  meta data storing area         */
/*               int  label_len  ->  access label length            */
/*               int  meta_len   ->  size of meta data name area    */
/*               int  value_len  ->  size og meta data storing area */
/*  return    :  int  -> data get status                            */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_get_meta_string_( int *p_hdf5_file_handle, char *label, char *meta, char *p_value,
                           int label_len, int meta_len, int value_len )
{
	int ret;
	char *ps;
	char *qs;
	char *q;
	int i,j;
	
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
    ret = SGTK_get_meta_string( *(p_hdf5_file_handle), ps, qs, q );
    if( ret > 0 )
    {
		j = strlen( q );
		for( i = 0; i < value_len; i++ )
		{
			*(p_value + i) = ' ';
			if( i < j )
			{
				*(p_value + i) = *(q + i);
			}
		}
	}
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_string__( int *p_hdf5_file_handle, char *label, char *meta, char *p_value,
                           int label_len, int meta_len, int value_len )
{
	return(sgtk_get_meta_string_( p_hdf5_file_handle, label, meta, p_value,
                           label_len, meta_len, value_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type int)                                     */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               int  *p_value  ->  meta data storing area      */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_int_( int *p_hdf5_file_handle, char *label, char *meta, int *p_value,
                        int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_int( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_int__( int *p_hdf5_file_handle, char *label, char *meta, int *p_value,
                        int label_len, int meta_len )
{
	return(sgtk_get_meta_int_( p_hdf5_file_handle, label, meta, p_value,
                        label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type int(1))                                  */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               int  *p_value  ->  meta data storing area      */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*               int8_t *value  ->  meta data storing area      */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_int8_( int *p_hdf5_file_handle, char *label, char *meta, int8_t *p_value,
                         int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_int8( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_int8__( int *p_hdf5_file_handle, char *label, char *meta, int8_t *p_value,
                         int label_len, int meta_len )
{
	return(sgtk_get_meta_int8_( p_hdf5_file_handle, label, meta, p_value,
                         label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type int(2))                                  */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               short  *p_value  ->  meta data storing area    */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_int16_( int *p_hdf5_file_handle, char *label, char *meta, int16_t *p_value,
                         int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_int16( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_int16__( int *p_hdf5_file_handle, char *label, char *meta, int16_t *p_value,
                         int label_len, int meta_len )
{
	return(sgtk_get_meta_int16_( p_hdf5_file_handle, label, meta, p_value,
                         label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type int(4))                                  */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               long  *p_value  ->  meta data storing area     */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_int32_( int *p_hdf5_file_handle, char *label, char *meta, int32_t *p_value,
                         int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_int32( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_int32__( int *p_hdf5_file_handle, char *label, char *meta, int32_t *p_value,
                         int label_len, int meta_len )
{
	return(sgtk_get_meta_int32_( p_hdf5_file_handle, label, meta, p_value,
                         label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  get meta data(type uint)                                        */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label     ->  dataset access label           */
/*               char *meta      ->  meta data name                 */
/*               uint  *p_value  ->  meta data storing area         */
/*               int  label_len  ->  access label length            */
/*               int  meta_len   ->  size of meta data name area    */
/*  return    :  int  -> data get status                            */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_get_meta_uint_( int *p_hdf5_file_handle, char *label, char *meta, uint *p_value,
                         int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_uint( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_uint__( int *p_hdf5_file_handle, char *label, char *meta, uint *p_value,
                         int label_len, int meta_len )
{
	return(sgtk_get_meta_uint_( p_hdf5_file_handle, label, meta, p_value,
                         label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type uint(1))                                  */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               int  *p_value  ->  meta data storing area      */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*               int8_t *value  ->  meta data storing area      */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_uint8_( int *p_hdf5_file_handle, char *label, char *meta, uint8_t *p_value,
                          int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_uint8( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_uint8__( int *p_hdf5_file_handle, char *label, char *meta, uint8_t *p_value,
                          int label_len, int meta_len )
{
	return(sgtk_get_meta_uint8_( p_hdf5_file_handle, label, meta, p_value,
                          label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type uint(2))                                  */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               short  *p_value  ->  meta data storing area    */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_uint16_( int *p_hdf5_file_handle, char *label, char *meta, uint16_t *p_value,
                           int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_uint16( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_uint16__( int *p_hdf5_file_handle, char *label, char *meta, uint16_t *p_value,
                           int label_len, int meta_len )
{
	return(sgtk_get_meta_uint16_( p_hdf5_file_handle, label, meta, p_value,
                           label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type uint(4))                                 */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               long  *p_value  ->  meta data storing area     */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_uint32_( int *p_hdf5_file_handle, char *label, char *meta, uint32_t *p_value,
                           int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_uint32( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_uint32__( int *p_hdf5_file_handle, char *label, char *meta, uint32_t *p_value,
                           int label_len, int meta_len )
{
	return(sgtk_get_meta_uint32_( p_hdf5_file_handle, label, meta, p_value,
                           label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type REAL(4))                                 */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               float *p_value  ->  meta data storing area     */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_float_( int *p_hdf5_file_handle, char *label, char *meta, float *p_value,
                          int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_float( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_float__( int *p_hdf5_file_handle, char *label, char *meta, float *p_value,
                          int label_len, int meta_len )
{
	return(sgtk_get_meta_float_( p_hdf5_file_handle, label, meta, p_value,
                          label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type REAL(4))                                 */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               float *p_value  ->  meta data storing area     */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_float32_( int *p_hdf5_file_handle, char *label, char *meta, float *p_value,
                          int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_float32( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_float32__( int *p_hdf5_file_handle, char *label, char *meta, float *p_value,
                          int label_len, int meta_len )
{
    return(sgtk_get_meta_float32_( p_hdf5_file_handle, label, meta, p_value,
                          label_len, meta_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  get meta data(type REAL(8))                                 */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label        */
/*               char *meta     ->  meta data name              */
/*               double *p_value  ->  meta data storing area     */
/*               int  label_len  -> access label length         */
/*               int  meta_len  ->  size of meta data name area */
/*  return    :  int  -> data get status                        */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_get_meta_float64_( int *p_hdf5_file_handle, char *label, char *meta, double *p_value,
                            int label_len, int meta_len )
{
    int ret;
    char *ps;
    char *qs;
    
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_get_meta_float64( *(p_hdf5_file_handle), ps, qs, p_value );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_get_meta_float64__( int *p_hdf5_file_handle, char *label, char *meta, double *p_value,
                            int label_len, int meta_len )
{
	return(sgtk_get_meta_float64_( p_hdf5_file_handle, label, meta, p_value,
                            label_len, meta_len ) );
}


/*--------------------------------------------------------------------*/
/*                                                                    */
/*  set a dataset by designating each dimension's size and label      */
/*  arguments :  int *p_hdf5_file_handle    -> opened file's handle   */
/*               char *dataset_access_label ->  dataset access label  */
/*               int  *p_dims               -> each dimension's size  */
/*               int  *p_rank               -> max dimensions         */
/*               int  *p_dat_type           -> dataset type           */
/*               char *p_dim_label          -> each dimension's label */
/*               int  label_len             -> access label length    */
/*               int  dim_label_len         -> dimension label length */
/*  return    :  int         -> set status                            */
/*                                                                    */
/*--------------------------------------------------------------------*/
int sgtk_set_dims_( int *p_hdf5_file_handle, char *dataset_access_label, int *p_dims,
                    int *p_rank, int *p_dat_type, char *dim_label, int label_len, int dim_label_len )
{
	int ret;
	int i;
	int rank;
	char *ps;
	char *q;
	char **qs;
	
	rank = *(p_rank);
    qs = (char **)malloc( rank * sizeof(q) );
	for (i = 0; i< rank; i++ )
	{
		*(qs + i) = SGTK_convstring_ftoc( (dim_label + dim_label_len * i), dim_label_len );
	}
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dims( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), (*p_dat_type), qs );
	for( i = 0; i < rank; i++)
	{
		free( *(qs + i) );
	}
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_set_dims__( int *p_hdf5_file_handle, char *dataset_access_label, int *p_dims,
                    int *p_rank, int *p_dat_type, char *dim_label, int label_len, int dim_label_len )
{
	return(sgtk_set_dims_( p_hdf5_file_handle, dataset_access_label, p_dims,
                    p_rank, p_dat_type, dim_label, label_len, dim_label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type int) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               int *p_put_value        -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_int_(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, int *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_int( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_int__(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, int *p_put_value, int label_len )
{
	return(sgtk_set_dataset_int_(p_hdf5_file_handle, dataset_access_label,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type int8) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               int8_t *p_put_value     -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_int8_(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, int8_t *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_int8( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_int8__(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, int8_t *p_put_value, int label_len )
{
	return(sgtk_set_dataset_int8_(p_hdf5_file_handle, dataset_access_label,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type int16) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               int6_t *p_put_value     -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_int16_(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, int16_t *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_int16( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_int16__(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, int16_t *p_put_value, int label_len )
{
	return(sgtk_set_dataset_int16_(p_hdf5_file_handle, dataset_access_label,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type int32) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               int32_t *p_put_value    -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_int32_(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, int32_t *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_int32( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_int32__(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, int32_t *p_put_value, int label_len )
{
	return(sgtk_set_dataset_int32_(p_hdf5_file_handle, dataset_access_label,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type uint) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               uint *p_put_value        -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_uint_(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, uint *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_uint( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_uint__(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, uint *p_put_value, int label_len )
{
	return(sgtk_set_dataset_uint_(p_hdf5_file_handle, dataset_access_label,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type uint8) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               int8_t *p_put_value     -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_uint8_(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, uint8_t *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_uint8( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_uint8__(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, uint8_t *p_put_value, int label_len )
{
	return(sgtk_set_dataset_uint8_(p_hdf5_file_handle, dataset_access_label,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type uint16) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               uint6_t *p_put_value    -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_uint16_(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, uint16_t *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_uint16( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_uint16__(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, uint16_t *p_put_value, int label_len )
{
	return(sgtk_set_dataset_uint16_(p_hdf5_file_handle, dataset_access_label,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type int32) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               int32_t *p_put_value    -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_uint32_(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, uint32_t *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_uint32( (*p_hdf5_file_handle), ps, p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_uint32__(int *p_hdf5_file_handle, char *dataset_access_label,
                          int *p_dims, int *p_rank, uint32_t *p_put_value, int label_len )
{
	return(sgtk_set_dataset_uint32_(p_hdf5_file_handle, dataset_access_label,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type REAL*4) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_kind             -> kind of values      */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               float *p_put_value      -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_float_(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind,
                          int *p_dims, int *p_rank, float *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_float( (*p_hdf5_file_handle), ps, (*p_kind), p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_float__(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind,
                          int *p_dims, int *p_rank, float *p_put_value, int label_len )
{
	return(sgtk_set_dataset_float_(p_hdf5_file_handle, dataset_access_label, p_kind,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type REAL*4) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_kind             -> kind of values      */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               float *p_put_value      -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_float32_(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind,
                          int *p_dims, int *p_rank, float *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_float32( (*p_hdf5_file_handle), ps, (*p_kind), p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_float32__(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind,
                          int *p_dims, int *p_rank, float *p_put_value, int label_len )
{
	return(sgtk_set_dataset_float32_(p_hdf5_file_handle, dataset_access_label, p_kind,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*--------------------------------------------------------------*/
/*                                                              */
/*  set data(type REAL*8) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle  */
/*               int *dataset_access_lbl -> dataset access label */
/*               int *p_kind             -> kind of values      */
/*               int *p_dims             -> each dimension's size  */
/*               int *p_rank             -> total dimensions    */
/*               double *p_put_value     -> data storing area   */
/*               int label_len           -> access label length */
/*  return    :  int         -> data set status                 */
/*                                                              */
/*--------------------------------------------------------------*/
int sgtk_set_dataset_float64_(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind,
                          int *p_dims, int *p_rank, double *p_put_value, int label_len )
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	ret = SGTK_set_dataset_float64( (*p_hdf5_file_handle), ps, (*p_kind), p_dims, (*p_rank), p_put_value);
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_float64__(int *p_hdf5_file_handle, char *dataset_access_label, int *p_kind,
                          int *p_dims, int *p_rank, double *p_put_value, int label_len )
{
	return(sgtk_set_dataset_float64_(p_hdf5_file_handle, dataset_access_label, p_kind,
                          p_dims, p_rank, p_put_value, label_len ) );
}


/*------------------------------------------------------------------------*/
/*                                                                        */
/*  set scanning data of latitudes and longitudes                         */
/*  arguments :  int *p_hdf5_file_handle -> opened file's handle          */
/*               char *latitude          -> latitude access label         */
/*               char *longitude         -> longitude access label        */
/*               int *p_kind             -> data kind                     */
/*               int *p_start_no         -> start scan no.                */
/*               int *p_end_no           -> end scan no.                  */
/*               int *p_set_num          -> the number of set data        */
/*               float *p_latlong        -> data storing area             */
/*               int  latlabel_lenn      -> latitude access label size    */
/*               int  lonlabel_len       -> longitude access label lenght */
/*  return    :  int         -> data get status                           */
/*                                                                        */
/*------------------------------------------------------------------------*/
int sgtk_set_dataset_latlon_(int *p_hdf5_file_handle, char *latitude, char *longitude, int *p_kind, int *p_start_no, int *p_end_no,
                             int *p_set_num, float *p_latlon, int latlabel_len, int lonlabel_len)
{
	int ret;
	char *ps1;
	char *ps2;
	
	ps1 = SGTK_convstring_ftoc( latitude, latlabel_len);
	ps2 = SGTK_convstring_ftoc( longitude, lonlabel_len);
	ret = SGTK_set_dataset_latlon( *(p_hdf5_file_handle), ps1, ps2, *(p_kind),
	                               *(p_start_no), *(p_end_no), *(p_set_num), p_latlon );
	free( ps1 );
	free( ps2 );
    return ret;
}

/* fortran77 */
int sgtk_set_dataset_latlon__(int *p_hdf5_file_handle, char *latitude, char *longitude, int *p_kind, int *p_start_no, int *p_end_no,
                             int *p_set_num, float *p_latlon, int latlabel_len, int lonlabel_len)
{
	return(sgtk_set_dataset_latlon_(p_hdf5_file_handle, latitude, longitude, p_kind, p_start_no, p_end_no,
                             p_set_num, p_latlon, latlabel_len, lonlabel_len) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set meta data(type string)                                      */
/*  arguments :  int *p_hdf5_file_handle   -> opened file's handle  */
/*               char *label    ->  dataset access label            */
/*               char *meta     ->  meta data name                  */
/*               char *p_value  ->  meta data storing area          */
/*               int  label_len  -> access label length             */
/*               int  meta_len  ->  size of meta data name area     */
/*               int  value_len ->  size og meta data storing area  */
/*  return    :  int  -> data get status                            */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_string_( int *p_hdf5_file_handle, char *label, char *meta, 
                           char *p_value, int label_len, int meta_len, int str_value_len )
{
	int ret;
	char *ps;
	char *qs;
	char *vp;
	
	ps = SGTK_convstring_ftoc( label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	vp = SGTK_convstring_ftoc( p_value, str_value_len );
	ret = SGTK_set_meta_string( *(p_hdf5_file_handle), ps, qs, vp );
    free( vp );
	free( qs );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_set_meta_string__( int *p_hdf5_file_handle, char *label, char *meta,
                            char *p_value, int label_len, int meta_len, int str_value_len )
{
	return(sgtk_set_meta_string_( p_hdf5_file_handle, label, meta,
                           p_value, label_len, meta_len, str_value_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( int ) of a dataset                                */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               int *p_dims              -> each dimension's size  */
/*               int *p_rank              -> total dimensions       */
/*               int *p_put_value         -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_int_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          int *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;

	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_int( (*p_hdf5_file_handle), ps, qs, *(p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_int__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          int *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_int_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( int8_t ) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               int8_t *p_put_value      -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_int8_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          int8_t *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_int8( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_int8__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          int8_t *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_int8_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( int16_t ) of a dataset                            */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               int16_t *p_put_value     -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_int16_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          int16_t *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_int16( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_int16__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          int16_t *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_int16_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( int32_t ) of a dataset                            */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               int32_t *p_put_value     -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_int32_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          int32_t *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_int32( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_int32__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          int32_t *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_int32_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( uint ) of a dataset                               */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               uint *p_put_value        -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_uint_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          uint *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_uint( (*p_hdf5_file_handle), ps, qs, *(p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_uint__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          uint *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_uint_(p_hdf5_file_handle, dataset_access_label, meta, 
                           p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( uint8_t ) of a dataset                            */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               uint8_t *p_put_value     -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_uint8_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          uint8_t *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_uint8( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_uint8__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          uint8_t *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_uint8_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( uint16_t ) of a dataset                           */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               uint16_t *p_put_value    -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_uint16_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          uint16_t *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_uint16( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_uint16__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                           uint16_t *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_uint16_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( uint32_t ) of a dataset                           */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               uint32_t *p_put_value    -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_uint32_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          uint32_t *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_uint32( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_uint32__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          uint32_t *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_uint32_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( float ) of a dataset                              */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               float *p_put_value       -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_float_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          float *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_float( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_float__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          float *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_float_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( REAL*4 ) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               float *p_put_value       -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_float32_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                           float *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_float32( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_float32__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                          float *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_float32_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}


/*------------------------------------------------------------------*/
/*                                                                  */
/*  set metadata( REAL*8 ) of a dataset                             */
/*  arguments :  int *p_hdf5_file_handle  -> opened file's handle   */
/*               char *dataset_access_lbl -> dataset access label   */
/*               char *meta               -> meta name              */
/*               double *p_put_value      -> data storing area      */
/*               int label_len            -> access label length    */
/*               int meta_len             -> meta name length       */
/*  return    :  int         -> data set status                     */
/*                                                                  */
/*------------------------------------------------------------------*/
int sgtk_set_meta_float64_(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                           double *p_put_value, int label_len, int meta_len )
{
	int ret;
	char *ps;
	char *qs;
	ps = SGTK_convstring_ftoc( dataset_access_label, label_len );
	qs = SGTK_convstring_ftoc( meta, meta_len );
	ret = SGTK_set_meta_float64( (*p_hdf5_file_handle), ps, qs, (*p_put_value) );
	free( qs );
	free( ps );
    return ret;
}

/* fortran77 */
int sgtk_set_meta_float64__(int *p_hdf5_file_handle, char *dataset_access_label, char *meta, 
                            double *p_put_value, int label_len, int meta_len )
{
	return(sgtk_set_meta_float64_(p_hdf5_file_handle, dataset_access_label, meta, 
                          p_put_value, label_len, meta_len ) );
}

/*** end of module ***/

int sgtk_make_parameter_table_(char *file_name, int flen)
{
	int ret;
	char *ps;
	ps = SGTK_convstring_ftoc( file_name, flen );
	ret = SGTK_make_parameter_table( &ps );
	free( ps );
	return ret;
}

/* fortran77 */
int sgtk_make_parameter_table__(char *file_name, int flen)
{
	return(sgtk_make_parameter_table_( file_name, flen ) );
}


